@@ -87,7 +87,7 @@ int adios_group_size (int64_t fd_p, uint64_t data_size
  * passing variable names, user is expected to pass variable id, which is returned
  * from adios_define_var call. Therefore, this function is only used in the no-xml way.
  */
-int adios_write_byid (int64_t fd_p, int64_t id, void * var)
+int adios_write_byid (int64_t fd_p, int64_t id, const void * var)
 {
     return common_adios_write_byid ((struct adios_file_struct *) fd_p, (struct adios_var_struct *) id, var);
 }
@@ -96,7 +96,7 @@ int adios_write_byid (int64_t fd_p, int64_t id, void * var)
 /* This C api function is a bit different from the Fortran api funcion, but
  * they call the same common_adios_write()
  */
-int adios_write (int64_t fd_p, const char * name, void * var)
+int adios_write (int64_t fd_p, const char * name, const void * var)
 {
     int retval;
     struct adios_file_struct * fd = (struct adios_file_struct *) fd_p;
@@ -273,6 +273,11 @@ int adios_close (int64_t fd_p)
 {
     int retval;
     struct adios_file_struct * fd = (struct adios_file_struct *) fd_p;
+    if (!fd)
+    {
+        adios_error (err_invalid_file_pointer, "Invalid handle passed to adios_close\n");
+        return adios_errno;
+    }
     struct adios_var_struct * v = fd->group->vars;
 
     retval = common_adios_close (fd_p);
@@ -433,6 +438,16 @@ int adios_delete_attrdefs (int64_t id)
     return adios_errno;
 }
 
+int adios_define_attribute_byvalue (int64_t group, 
+                            const char * name, const char * path, 
+                            enum ADIOS_DATATYPES type, int  nelems, void * values
+                           )
+{
+    adios_errno = err_no_error;
+    adios_common_define_attribute_byvalue (group, name, path, type, nelems, values);
+    return adios_errno;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 
 // adios_common_select_method is in adios_internals_mxml.c
