@@ -63,7 +63,8 @@ struct adios_var_struct
     uint64_t write_offset;  // offset this var was written at  [for writes]
 
     enum ADIOS_FLAG free_data;    // primarily used for writing
-    void * data;                  // primarily used for reading
+    const void * data;           // user data pointer 
+    void * adata;                // data field if allocated inside adios (then, data->adata)
     uint64_t data_size;           // primarily used for reading
     uint32_t write_count; // added to support multiple writes for transform layer.
                           // Might needed for other things in the future.
@@ -100,10 +101,13 @@ struct adios_attribute_struct
     char * name;
     char * path;
     enum ADIOS_DATATYPES type;
+    int    nelems; // number of elements of 'type' in the attribute
     void * value;
     struct adios_var_struct * var;
     uint64_t write_offset;  // offset this var was written at  [for writes]
 
+    uint32_t data_size; // data size for an array of strings (type==adios_string_array)
+
     struct adios_attribute_struct * next;
 };
 
@@ -204,7 +208,7 @@ struct adios_dimension_item_struct
     uint64_t rank;                 // for numerical value
     struct adios_var_struct * var; // for value stored in variable
     struct adios_attribute_struct * attr; // for value stored in attribute
-    enum ADIOS_FLAG time_index;
+    enum ADIOS_FLAG is_time_index;
 };
 
 struct adios_dimension_struct
@@ -323,7 +327,7 @@ typedef enum ADIOS_FLAG (* ADIOS_SHOULD_BUFFER_FN)
                                        );
 typedef void (* ADIOS_WRITE_FN) (struct adios_file_struct * fd
                                 ,struct adios_var_struct * v
-                                ,void * data
+                                ,const void * data
                                 ,struct adios_method_struct * method
                                 );
 typedef void (* ADIOS_GET_WRITE_BUFFER_FN) (struct adios_file_struct * fd
@@ -435,7 +439,8 @@ int adios_common_define_attribute (int64_t group, const char * name
 int adios_common_define_attribute_byvalue (int64_t group, const char * name
                                   ,const char * path
                                   ,enum ADIOS_DATATYPES type
-                                  ,void * value
+                                  ,int nelems
+                                  ,void * values
                                   );
 
 void adios_append_method (struct adios_method_struct * method);
@@ -461,7 +466,7 @@ int adios_common_declare_group (int64_t * id, const char * name
                                ,enum ADIOS_FLAG host_language_fortran
                                ,const char * coordination_comm
                                ,const char * coordination_var
-                               ,const char * time_index
+                               ,const char * time_index_name
                                ,enum ADIOS_FLAG stats
                                );
 
@@ -550,26 +555,40 @@ void adios_merge_index_v1 (
                   ,struct adios_index_process_group_struct_v1 * new_pg_root
                   ,struct adios_index_var_struct_v1 * new_vars_root
                   ,struct adios_index_attribute_struct_v1 * new_attrs_root
+                  ,int needs_sorting // merge-sort the characteristics to keep the time in order
                   );
 
+/* obsolete, merge the index with sorting
 void adios_sort_index_v1 (struct adios_index_process_group_struct_v1 ** p1
                          ,struct adios_index_var_struct_v1 ** v1
                          ,struct adios_index_attribute_struct_v1 ** a1
                          );
+*/
  
 void adios_clear_index_v1 (struct adios_index_struct_v1 * index); // in each adios_<method>_close()
 void adios_free_index_v1 (struct adios_index_struct_v1 * index);  // in adios_<method>_finalize()
 
+/* Other index operations made visible for the recovery tool */
+void index_append_process_group_v1 ( 
+            struct adios_index_struct_v1 * index, 
+            struct adios_index_process_group_struct_v1 * item
+            );
+void index_append_var_v1 (
+        struct adios_index_struct_v1 *index,
+        struct adios_index_var_struct_v1 * item,
+        int do_sort
+        );
+
 int adios_parse_scalar_string (enum ADIOS_DATATYPES type, char * value, void ** out);
 
 // NCSU ALACRITY-ADIOS - This function was static, but is now needed in adios_transforms_*.c
 uint8_t count_dimensions (const struct adios_dimension_struct * dimensions);
 
-uint64_t adios_get_type_size (enum ADIOS_DATATYPES type, void * var);
+uint64_t adios_get_type_size (enum ADIOS_DATATYPES type, const void * var);
 // NCSU ALACRITY-ADIOS - added this for use in the transform layer
 uint64_t adios_get_dimension_space_size (struct adios_var_struct * var
                                         ,struct adios_dimension_struct * d);
-uint64_t adios_get_var_size (struct adios_var_struct * var, void * data);
+uint64_t adios_get_var_size (struct adios_var_struct * var, const void * data);
 uint64_t adios_get_dim_value (struct adios_dimension_item_struct * dimension);
 uint64_t adios_get_stat_size (void * data, enum ADIOS_DATATYPES type, enum ADIOS_STAT stat_id);
 uint8_t adios_get_stat_set_count (enum ADIOS_DATATYPES type);
