@@ -238,10 +238,10 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
     }
     */
 
-    if (v->data)
+    if (v->adata)
     {
-        free (v->data);
-        v->data = 0;
+        free (v->adata);
+        v->adata = 0;
     }
 
     // Q.L. 10-2010. To fix a memory leak problem.
@@ -305,8 +305,8 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
             case adios_long_double:
             case adios_complex:
             case adios_double_complex:
-                v->data = malloc (element_size);
-                if (!v->data)
+                v->adata = malloc (element_size);
+                if (!v->adata)
                 {
                     adios_error (err_no_memory, 
                                  "In adios_write, cannot allocate %lld bytes to copy scalar %s\n",
@@ -316,11 +316,12 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
                     return;
                 }
 
-                memcpy ((char *) v->data, var, element_size);
+                memcpy ((char *) v->adata, var, element_size);
+                v->data = v->adata;
                 break;
             case adios_string:
-                v->data = futils_fstr_to_cstr (var, var_size);
-                if (!v->data)
+                v->adata = futils_fstr_to_cstr (var, var_size);
+                if (!v->adata)
                 {
                     adios_error (err_no_memory, 
                                  "In adios_write, cannot allocate %lld bytes to copy string %s\n",
@@ -329,6 +330,7 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
                     free (buf1);
                     return;
                 }
+                v->data = v->adata;
                 break;
 
             default:
@@ -338,7 +340,7 @@ void FC_FUNC_(adios_write, ADIOS_WRITE)
     }
 
     *err = common_adios_write (fd, v, var);
-    if (fd->mode == adios_mode_write || fd->mode == adios_mode_append)
+    if (fd->mode != adios_mode_read)
     {
         adios_copy_var_written (fd->group, v);
     }
@@ -614,6 +616,96 @@ void FC_FUNC_(adios_define_attribute, ADIOS_DEFINE_ATTRIBUTE)
     *err = adios_errno;
 }
 
+void FC_FUNC_(adios_define_attribute_byvalue, ADIOS_DEFINE_ATTRIBUTE_BYVALUE) 
+    (int64_t * group, const char * name, const char * path, int * type, int * nelems, 
+     void * values, int * err, int name_size, int path_size)
+{
+    char * buf1 = 0;
+    char * buf2 = 0;
+
+    adios_errno = err_no_error;
+    buf1 = futils_fstr_to_cstr (name, name_size);
+    buf2 = futils_fstr_to_cstr (path, path_size);
+
+    if (buf1 != 0 && buf2 != 0) {
+        adios_common_define_attribute_byvalue (*group, buf1, buf2
+                                      ,(enum ADIOS_DATATYPES) *type, *nelems, values);
+        free (buf1);
+        free (buf2);
+    }
+    *err = adios_errno;
+}
+
+/* Name clash resolution: Fortran adios_define_attribute_byvalue is an interface, and its 
+   subroutines cannot call adios_define_attribute_byvalue() in this file directly because 
+   the Fortran compiler interprets it as the call to the interface name. 
+   adios_define_attribute_byvalue_f2c provides the bridge to link the C function with
+   the subroutines. 
+*/
+void FC_FUNC_(adios_define_attribute_byvalue_f2c, ADIOS_DEFINE_ATTRIBUTE_BYVALUE_F2C) 
+    (int64_t * group, const char * name, const char * path, int * type, int * nelems, 
+     void * values, int * err, int name_size, int path_size)
+{
+    FC_FUNC_(adios_define_attribute_byvalue, ADIOS_DEFINE_ATTRIBUTE_BYVALUE) 
+         (group, name, path, type, nelems, values, err, name_size, path_size);
+}
+
+void FC_FUNC_(adios_define_attribute_byvalue_string, ADIOS_DEFINE_ATTRIBUTE_BYVALUE_STRING) 
+    (int64_t * group, const char * name, const char * path, int * nelems, 
+     char * values, int * err, int name_size, int path_size, int values_size)
+{
+    char * valuestr = futils_fstr_to_cstr (values, values_size);
+    int type = adios_string;
+
+    if (valuestr != 0 && *nelems == 1) {
+        FC_FUNC_(adios_define_attribute_byvalue, ADIOS_DEFINE_ATTRIBUTE_BYVALUE) 
+             (group, name, path, &type, nelems, valuestr, err, name_size, path_size);
+        free (valuestr);
+    } else {
+        if (*nelems != 1) {
+            adios_error (err_corrupted_attribute, 
+                    "String attributes can only store a single string. "
+                    "For multiple strings (nelems>1), use adios_string_array type\n");
+        } else {
+            adios_error (err_corrupted_attribute, 
+                    "adios_define_attribute_byvalue(string) called with a string "
+                    "that could not be processed in the C library\n");
+        }
+        *err = adios_errno;
+    }
+}
+
+void FC_FUNC_(adios_define_attribute_byvalue_string_array, ADIOS_DEFINE_ATTRIBUTE_BYVALUE_STRING_ARRAY) 
+    (int64_t * group, const char * name, const char * path, int * nelems, 
+     char * values, int * string_size, int * err, int name_size, int path_size, int values_size)
+{
+    /* *string_size = the size of each string in the string array,
+                      = i.e. XXX in  "character(len=XXX), dimension (YYY) :: values"
+       Probably with all compilers, or most, it is also true that values_size = *string_size.
+       'values'  here in the C code is a single byte array storing the strings contiguously 
+    */
+    int k;
+    int type = adios_string_array;
+
+    //printf ("C String Array: nelems=%d, size=%d\n", *nelems, *string_size);
+    char ** strings = (char**) malloc (*nelems * sizeof (char*));
+    char *p = values;
+    if (strings != 0) {
+        for (k=0; k < *nelems; k++) {
+            //printf ("  k=%d, ptr=%p size=%d\n", k, p, *string_size);
+            strings[k] = futils_fstr_to_cstr (p, *string_size);
+            p += *string_size;
+        }
+
+        FC_FUNC_(adios_define_attribute_byvalue, ADIOS_DEFINE_ATTRIBUTE_BYVALUE) 
+             (group, name, path, &type, nelems, strings, err, name_size, path_size);
+        free (strings);
+    } else {
+        adios_error (err_corrupted_attribute, "adios_define_attribute_byvalue(string) called with a string that could not be processed in the C library\n");
+        *err = adios_errno;
+    }
+}
+
 // delete all attribute definitions from a group
 // Use if you want to define a new set of attribute for the next output step.
 void FC_FUNC_(adios_delete_attrdefs, ADIOS_DELETE_VARDEFS) (int64_t *id, int *err)
