@@ -9,7 +9,7 @@
 #include "public/adios_error.h"
 #include "core/common_read.h"
 #include "core/adios_logger.h"
-
+#include "query_utils.h"
 static struct adios_query_hooks_struct * query_hooks = 0;
 
 static int getTotalByteSize (ADIOS_FILE* f, ADIOS_VARINFO* v, ADIOS_SELECTION* sel, 
@@ -45,27 +45,27 @@ static ADIOS_SELECTION* getAdiosDefaultBoundingBox(ADIOS_VARINFO* v)
   return result;
 }
 
+static int query_hooks_initialized = 0;
 void common_query_init()
 {
-    static int did_init = 0;
-    if (!did_init) {
+    if (!query_hooks_initialized) {
         adios_query_hooks_init(&query_hooks);
-        did_init = 1;
+        query_hooks_initialized = 1;
     }
 }
 
 void common_query_finalize()
 {
-    static int did_finalize = 0;
-    if (!did_finalize) {
+    if (query_hooks_initialized) {
         enum ADIOS_QUERY_METHOD m;
         for (m=0; m < ADIOS_QUERY_METHOD_COUNT; m++) {
 	  if (query_hooks[m].adios_query_finalize_fn != NULL) {
 	    query_hooks[m].adios_query_finalize_fn();
 	  }
         }
-        free(query_hooks);
-        did_finalize = 1;
+        // Do not free query_hooks here because they are initialized only once
+        // in common_query_init ---> adios_query_hooks_init()
+        query_hooks_initialized = 0;
     }
 }
 
@@ -164,6 +164,12 @@ static int adios_check_query_at_timestep(ADIOS_QUERY* q, int timeStep)
 	  timeStep = q->file->current_step;
       }
 
+      if (q->varinfo != NULL) {
+	if (q->onTimeStep  == timeStep) {
+	  return timeStep; // returning call to get more values
+	}
+      }
+
       ADIOS_VARINFO* v = common_read_inq_var(q->file, q->varName);
       if (v == NULL) {
 	adios_error (err_invalid_varname, "Query Invalid variable '%s':\n%s",
@@ -184,8 +190,9 @@ static int adios_check_query_at_timestep(ADIOS_QUERY* q, int timeStep)
 	return -1;
       }
 
-      log_debug("%s, raw data size=%ld\n", q->condition, dataSize);
-      q->dataSlice = malloc(total_byte_size);
+      log_debug("%s, raw data size=%llu\n", q->condition, dataSize);
+      //q->dataSlice = malloc(total_byte_size);
+      q->dataSlice = 0;
       q->rawDataSize = dataSize;
 
       return timeStep;
@@ -202,6 +209,7 @@ static int adios_check_query_at_timestep(ADIOS_QUERY* q, int timeStep)
 
 	return -1;
       }
+      q->rawDataSize = ((ADIOS_QUERY*)(q->left))->rawDataSize;
       return leftTimeStep;
     }
 }
@@ -459,7 +467,7 @@ static int isSelectionCompatible(ADIOS_SELECTION* first, ADIOS_SELECTION* second
     const ADIOS_SELECTION_POINTS_STRUCT *pt2 = &(second->u.points);
     
     if (pt1 -> npoints != pt2->npoints) {
-      log_error("Error! point selections have different size. %ld != %ld\n", pt1->npoints, pt2->npoints);
+      log_error("Error! point selections have different size. %llu != %llu\n", pt1->npoints, pt2->npoints);
       return -1;
     }
     return 1;
@@ -547,6 +555,7 @@ ADIOS_QUERY* common_query_combine(ADIOS_QUERY* q1,
     result->right = q2;
     result->combineOp = operator;
 
+    result->rawDataSize = q1->rawDataSize;
     //initialize(result);
     return result;
 }
@@ -768,6 +777,10 @@ int common_query_evaluate(ADIOS_QUERY* q,
 			  uint64_t batchSize, // limited by maxResult
 			  ADIOS_SELECTION** result)
 {  
+	double start = 0, end = 0;
+#ifdef BREAKDOWN
+	start = dclock();
+#endif
   if (q == 0) {
     log_debug("Error: empty query will not be evaluated!");
     return -1;
@@ -816,9 +829,20 @@ int common_query_evaluate(ADIOS_QUERY* q,
     if (query_hooks[m].adios_query_evaluate_fn != NULL) {
       int retval = query_hooks[m].adios_query_evaluate_fn(q, timeStep, batchSize, outputBoundary, result);	      
       if (freeOutputBoundary) common_read_selection_delete(outputBoundary);
+
+#ifdef BREAKDOWN
+      end = dclock();
+      printf("total time [frame + plugin + adios] : %f \n", end - start);
+#endif
       return retval;
     } 
     log_debug ("No selection method is supported for method: %d\n", m);
+
+#ifdef BREAKDOWN
+      end = dclock();
+      printf("total time [frame + plugin + adios] : %f \n", end - start);
+#endif
+
     return -1;
 }
 
