@@ -7,6 +7,264 @@
 
 #include "fastbit_adios.h"
 
+const char* _prefix="";
+
+unsigned long _startMillis = 0;
+unsigned long _milestoneMillis = 0;
+
+//struct timespec _bmsVisits;
+//struct timespec _idxVisits;
+
+CollectionPoint _bmsPoint;
+CollectionPoint _idxPoint;
+CollectionPoint _proPoint; // fastbit_evaluate & getCoordinate
+CollectionPoint _framePoint;
+
+void casestudyLogger_initPoints(CollectionPoint* p) 
+{
+  p->_accumulator.tv_nsec = 0;
+  p->_accumulator.tv_sec = 0;
+
+  p->_counter = 0;
+}
+
+void casestudyLogger_init() 
+{
+  casestudyLogger_initPoints(&_bmsPoint);
+  casestudyLogger_initPoints(&_idxPoint);
+  casestudyLogger_initPoints(&_proPoint);
+  casestudyLogger_initPoints(&_framePoint);
+}
+
+extern void casestudyLogger_print(CollectionPoint* p, const char* msg)
+{
+#ifdef TIMESTUDY
+  printf("%s: %llu sec %llu millisec  accumulated, visited: %lu times \n",  msg,  (unsigned long long)p->_accumulator.tv_sec, (unsigned long long)p->_accumulator.tv_nsec/((unsigned long)1000000), p->_counter);
+#else
+  log_debug("%s: %llu sec %llu millisec  accumulated, visited: %lu times \n",  msg,  (unsigned long long)p->_accumulator.tv_sec, (unsigned long long)p->_accumulator.tv_nsec/((unsigned long)1000000), p->_counter);
+#endif
+}
+
+extern void casestudyLogger_starts(const char* ref)
+{
+  _startMillis = fastbit_adios_getCurrentTimeMillis();
+  _milestoneMillis = 0;
+
+  /*_bmsVisits.tv_nsec=0;
+  _bmsVisits.tv_sec=0;
+  _idxVisits.tv_nsec=0;
+  _idxVisits.tv_sec=0;
+  */
+#ifdef TIMESTUDY
+  printf("==> casestudy %s starts at: %llu sec\n", ref, _startMillis/1000);
+#endif
+}
+
+
+extern void casestudyLogger_bms_print()
+{
+  //printf("bitmap visit took: %llu sec %llu millisec  accumulated \n",  _bmsVisits.tv_sec, _bmsVisits.tv_nsec/((unsigned long)1000000));
+  casestudyLogger_print(&_bmsPoint, "bitmap visit took");
+}
+
+extern void casestudyLogger_idx_print()
+{
+  //printf("idx access took: %llu sec %llu millisec  accumulated \n",  _idxVisits.tv_sec, _idxVisits.tv_nsec/((unsigned long)1000000));
+  casestudyLogger_print(&_idxPoint, "idx access took:");
+}
+
+extern void casestudyLogger_pro_print()
+{
+  //printf("idx access took: %llu sec %llu millisec  accumulated \n",  _idxVisits.tv_sec, _idxVisits.tv_nsec/((unsigned long)1000000));
+  casestudyLogger_print(&_proPoint, "fastbit query process took:");
+}
+
+extern void casestudyLogger_frame_print()
+{
+  //printf("idx access took: %llu sec %llu millisec  accumulated \n",  _idxVisits.tv_sec, _idxVisits.tv_nsec/((unsigned long)1000000));
+  casestudyLogger_print(&_framePoint, "query frame took:");
+}
+
+extern void casestudyLogger_ends(const char* ref)
+{
+  unsigned long endMillis = fastbit_adios_getCurrentTimeMillis();
+  unsigned long diffmillis = endMillis - _startMillis;
+
+#ifdef TIMESTUDY
+  unsigned long diffsec = diffmillis/1000;
+  if (diffsec > 60) {
+    unsigned long diffmin = diffsec/60;
+    diffsec = diffsec - diffmin*60;
+    printf("%s took: %lu millisecs = %lu min & %lu sec \n", ref, diffmillis, diffmin, diffsec);
+  } else {
+    printf("%s took: %lu millisecs = %lu sec\n", ref, diffmillis, diffsec);
+  }
+#endif
+}
+
+extern void casestudyLogger_getRealtime(struct timespec* spec)
+{
+#ifdef CLOCK_MONOTONIC
+  clock_gettime(CLOCK_MONOTONIC, spec);
+#else
+  clock_gettime(CLOCK_REALTIME, spec);
+#endif
+}
+
+extern void casestudyLogger_setPrefix(const char* prefix)
+{
+  _prefix = prefix;
+
+  unsigned long currMillis = fastbit_adios_getCurrentTimeMillis();
+  unsigned long diffMillis = currMillis-_startMillis;
+  unsigned long diffsec = diffMillis/1000;
+
+  unsigned long ministepMillis = currMillis - _milestoneMillis;
+  if (_milestoneMillis == 0) {
+    ministepMillis = diffMillis;
+  } 
+    
+
+#ifdef TIMESTUDY
+  if (diffsec > 60) {
+    unsigned long diffmin = diffsec/60;
+    diffsec = diffsec - diffmin*60;
+    printf("on %s, time passed since start: %lu millisecs = %lu min & %lu sec, ministep=%lu milliseconds \n", prefix, diffMillis, diffmin, diffsec, ministepMillis);
+  } else {
+    printf("on %s, time passed since start: %lu millisecs = %lu sec, ministep=%lu milliseconds\n", prefix, diffMillis, diffsec, ministepMillis);
+  }
+#endif
+  _milestoneMillis = currMillis;
+}
+
+/*
+extern void casestudyLogger_idx_writeout(struct timespec* start, 
+					 //struct timespec* end,
+					 const char* desc)
+{
+  struct timespec end; 
+  casestudyLogger_getRealtime(&end);
+
+  struct timespec diff; 
+
+  diff.tv_nsec = end.tv_nsec - start->tv_nsec; 
+  diff.tv_sec = end.tv_sec - start->tv_sec; 
+
+  if(diff.tv_nsec < 0 ){
+    diff.tv_nsec += 1000000000; 
+    diff.tv_sec = diff.tv_sec - 1;
+  } 
+
+  //printf("%s %s %llu sec %llu nanosec \n", _prefix, desc, diff.tv_sec, diff.tv_nsec);
+
+  _idxVisits.tv_sec  += diff.tv_sec;
+  _idxVisits.tv_nsec += diff.tv_nsec;
+
+  unsigned long sec = _idxVisits.tv_nsec/(unsigned long)1000000000;
+  if (sec > 0) {
+    _idxVisits.tv_sec += sec;
+    _idxVisits.tv_nsec -= sec*((unsigned long)1000000000);
+  } 
+
+}
+
+
+extern void casestudyLogger_bms_writeout(struct timespec* start, 
+                              //struct timespec* end,
+			      const char* desc)
+{
+  struct timespec end; 
+  casestudyLogger_getRealtime(&end);
+
+  struct timespec diff; 
+
+  diff.tv_nsec = end.tv_nsec - start->tv_nsec; 
+  diff.tv_sec = end.tv_sec - start->tv_sec; 
+
+  if(diff.tv_nsec < 0 ){
+    diff.tv_nsec += 1000000000; 
+    diff.tv_sec = diff.tv_sec - 1;
+  } 
+
+  //printf("%s %s %llu sec %llu nanosec \n", _prefix, desc, diff.tv_sec, diff.tv_nsec);
+
+  _bmsVisits.tv_sec  += diff.tv_sec;
+  _bmsVisits.tv_nsec += diff.tv_nsec;
+
+  unsigned long sec = _bmsVisits.tv_nsec/(unsigned long)1000000000;
+  if (sec > 0) {
+    _bmsVisits.tv_sec += sec;
+    _bmsVisits.tv_nsec -= sec*((unsigned long)1000000000);
+  } 
+  //printf("%s %s %llu sec %llu millisec  accumulated \n", _prefix, desc, _bmsVisits.tv_sec, _bmsVisits.tv_nsec/((unsigned long)1000000));
+}
+*/
+
+extern void casestudyLogger_writeout(CollectionPoint* p,
+				     struct timespec* start, 
+				     const char* desc)
+{
+  struct timespec end; 
+  casestudyLogger_getRealtime(&end);
+
+  struct timespec diff; 
+
+  diff.tv_nsec = end.tv_nsec - start->tv_nsec; 
+  diff.tv_sec = end.tv_sec - start->tv_sec; 
+
+  if(diff.tv_nsec < 0 ){
+    diff.tv_nsec += 1000000000; 
+    diff.tv_sec = diff.tv_sec - 1;
+  } 
+
+  //printf("%s %s %llu sec %llu nanosec \n", _prefix, desc, diff.tv_sec, diff.tv_nsec);
+
+  p->_accumulator.tv_sec  += diff.tv_sec;
+  p->_accumulator.tv_nsec += diff.tv_nsec;
+
+  unsigned long sec = p->_accumulator.tv_nsec/(unsigned long)1000000000;
+  if (sec > 0) {
+    p->_accumulator.tv_sec += sec;
+    p->_accumulator.tv_nsec -= sec*((unsigned long)1000000000);
+  } 
+  //printf("%s %s %llu sec %llu millisec  accumulated \n", _prefix, desc, _bmsVisits.tv_sec, _bmsVisits.tv_nsec/((unsigned long)1000000));
+  
+  p->_counter ++;
+}
+
+extern void casestudyLogger_idx_writeout(struct timespec* start, 
+					 //struct timespec* end,
+					 const char* desc)
+{
+  casestudyLogger_writeout(&_idxPoint, start, desc);
+}
+
+extern void casestudyLogger_pro_writeout(struct timespec* start, 
+					 //struct timespec* end,
+					 const char* desc)
+{
+  casestudyLogger_writeout(&_proPoint, start, desc);
+}
+
+extern void casestudyLogger_frame_writeout(struct timespec* start, 
+					 //struct timespec* end,
+					 const char* desc)
+{
+  casestudyLogger_writeout(&_framePoint, start, desc);
+  //printf("%s %llu sec %llu millisec  accumulated \n", desc, _framePoint._accumulator.tv_sec, _framePoint._accumulator.tv_nsec/((unsigned long)1000000));
+}
+
+extern void casestudyLogger_bms_writeout(struct timespec* start, 
+					 //struct timespec* end,
+					 const char* desc)
+{
+  casestudyLogger_writeout(&_bmsPoint, start, desc);
+}
+
+
+//
+//
+//
 long fastbit_adios_getCurrentTimeMillis() 
 {
   time_t          s;  // Seconds
@@ -48,8 +306,8 @@ int fastbit_adios_util_getRelativeBlockNumForPoint(ADIOS_VARINFO* v,  uint64_t*
 	
       ADIOS_VARBLOCK curr = v->blockinfo[i];
       for (j=0; j<v->ndim; j++) {
-	int begin = curr.start[j];
-	int end   = curr.start[j]+curr.count[j];
+	uint64_t begin = curr.start[j];
+	uint64_t end   = curr.start[j]+curr.count[j];
 
 	if ((begin <= point[j]) && (point[j] < end)) {
 	  result = i; // relative to the timestep                                                                                                                                      
@@ -253,6 +511,10 @@ uint64_t fastbit_adios_util_getBlockSize(ADIOS_VARINFO* v, int timestep, int rel
 
   k += relativeBlockIdx;
   
+  if (k > v->sum_nblocks-1) {
+    return 0;
+  }
+
   log_debug("\n blockinfo[%d]: [ ", k);
   
   for (j=0; j<v->ndim; j++) 
@@ -339,6 +601,25 @@ static const char * value_to_string (enum ADIOS_DATATYPES type, void * data, int
   return s;
 }
 
+void getVarNames(char* bmsVarName, char* keyVarName, char* offsetName, ADIOS_VARINFO* v, int timestep, int blockNum) {
+
+  /*
+  sprintf(bmsVarName, "bms-%d-%d-block-%d", v->varid, timestep, blockNum);
+  sprintf(keyVarName, "key-%d-%d-block-%d", v->varid, timestep, blockNum);
+  sprintf(offsetName, "offset-%d-%d-block-%d", v->varid, timestep, blockNum);
+  */
+
+  sprintf(bmsVarName, "bms-%d-%d-box-%d", v->varid, timestep, blockNum);
+  sprintf(keyVarName, "key-%d-%d-box-%d", v->varid, timestep, blockNum);
+  sprintf(offsetName, "offset-%d-%d-box-%d", v->varid, timestep, blockNum);
+
+  /*
+  sprintf(bmsVarName, "bms-%d-%d-%d", v->varid, timestep, blockNum);
+  sprintf(keyVarName, "key-%d-%d-%d", v->varid, timestep, blockNum);
+  sprintf(offsetName, "offset-%d-%d-%d", v->varid, timestep, blockNum);
+  */
+
+}
 //
 //
 // caller frees keys, offsets and bms.
@@ -352,10 +633,14 @@ int fastbit_adios_util_readNoBMSFromIndexFile(ADIOS_FILE* idxFile, ADIOS_VARINFO
   char offsetName[100];
 
   *bmsVarName = malloc(100);
+
+  getVarNames(*bmsVarName, keyVarName, offsetName, v, timestep, blockNum);
+
+  /*
   sprintf(*bmsVarName, "bms-%d-%d-%d", v->varid, timestep, blockNum);
   sprintf(keyVarName, "key-%d-%d-%d", v->varid, timestep, blockNum);
   sprintf(offsetName, "offset-%d-%d-%d", v->varid, timestep, blockNum);
-
+  */
   log_debug("reading from index file: %s for variables: %s %s %s \n", idxFile->path, *bmsVarName, keyVarName, offsetName);
 
   ADIOS_VARINFO * keyV = common_read_inq_var (idxFile, keyVarName);
@@ -409,10 +694,12 @@ int fastbit_adios_util_readFromIndexFile(ADIOS_FILE* idxFile, ADIOS_VARINFO* v,
   char keyVarName[100];
   char offsetName[100];
 
+  getVarNames(bmsVarName, keyVarName,  offsetName, v, timestep, blockNum);
+  /*
   sprintf(bmsVarName, "bms-%d-%d-%d", v->varid, timestep, blockNum);
   sprintf(keyVarName, "key-%d-%d-%d", v->varid, timestep, blockNum);
   sprintf(offsetName, "offset-%d-%d-%d", v->varid, timestep, blockNum);
-
+  */
   log_debug("reading from index file: %s for variables: %s %s %s \n", idxFile->path, bmsVarName, keyVarName, offsetName);
 
   ADIOS_VARINFO * bmsV = common_read_inq_var (idxFile, bmsVarName);
