@@ -287,7 +287,7 @@ enum ADIOS_FLAG adios_dataspaces_should_buffer (struct adios_file_struct * fd
 
 void adios_dataspaces_write (struct adios_file_struct * fd
                       ,struct adios_var_struct * v
-                      ,void * data
+                      ,const void * data
                       ,struct adios_method_struct * method
                       )
 {
@@ -425,11 +425,11 @@ void adios_dataspaces_get_write_buffer (struct adios_file_struct * fd
         return;
     }
 
-    if (v->data && v->free_data == adios_flag_yes)
+    if (v->adata && v->free_data == adios_flag_yes)
     {
         adios_method_buffer_free (v->data_size);
-        free (v->data);
-        v->data = NULL;
+        free (v->adata);
+        v->data = v->adata = NULL;
     }
 
     mem_allowed = adios_method_buffer_alloc (*size);
@@ -544,7 +544,7 @@ static void adios_dataspaces_gather_indices (struct adios_file_struct * fd
                         ,&new_attrs_root
                         );
                 adios_merge_index_v1 (index, new_pg_root, 
-                                      new_vars_root, new_attrs_root);
+                                      new_vars_root, new_attrs_root, 0);
                 new_pg_root = 0;
                 new_vars_root = 0;
                 new_attrs_root = 0;
