@@ -192,9 +192,9 @@ char*
 resolve_path_name(char *path, char *name)
 {
     char *fullname = NULL;
-    if(name){
-        if(path){
-            if(strcmp(path, "")){
+    if (name) {
+        if (path) {
+            if (strcmp(path, "")) {
                 fullname = malloc(strlen(path) + strlen(name) + 2);
                 strcpy(fullname, path);
                 strcat(fullname, "/");
@@ -238,7 +238,7 @@ set_flush_id_atom(attr_list attrs, int value)
 {
     atom_t dst_atom = attr_atom_from_string("fp_flush_id");
     int dst;
-    if(!get_int_attr(attrs, dst_atom, &dst)) {
+    if (!get_int_attr(attrs, dst_atom, &dst)) {
         add_int_attr(attrs, dst_atom, value);
     }
     set_int_attr(attrs, dst_atom, value);
@@ -251,7 +251,7 @@ set_size_atom(attr_list attrs, int value)
 {
     atom_t dst_atom = attr_atom_from_string("fp_size");
     int size;
-    if(!get_int_attr(attrs, dst_atom, &size)) {
+    if (!get_int_attr(attrs, dst_atom, &size)) {
         add_int_attr(attrs, dst_atom, value);
     }
     set_int_attr(attrs, dst_atom, value);
@@ -264,7 +264,7 @@ set_dst_rank_atom(attr_list attrs, int value)
 {
     atom_t dst_atom = attr_atom_from_string("fp_dst_rank");
     int dst;
-    if(!get_int_attr(attrs, dst_atom, &dst)) {
+    if (!get_int_attr(attrs, dst_atom, &dst)) {
         add_int_attr(attrs, dst_atom, value);
     }
     set_int_attr(attrs, dst_atom, value);
@@ -277,7 +277,7 @@ set_dst_condition_atom(attr_list attrs, int condition)
 {
     atom_t dst_atom = attr_atom_from_string("fp_dst_condition");
     int dst;
-    if(!get_int_attr(attrs, dst_atom, &dst)){
+    if (!get_int_attr(attrs, dst_atom, &dst)) {
 	add_int_attr(attrs, dst_atom, condition);
     }
     set_int_attr(attrs, dst_atom, condition);
@@ -293,7 +293,7 @@ evgroup_msg_free(void *eventData, void *clientData)
     /* evgroup *msg = (evgroup*)eventData; */
     /* int num_vars = msg->num_vars; */
     /* int i; */
-    /* for(i=0; i<num_vars; i++){ */
+    /* for (i=0; i<num_vars; i++) { */
     /* 	free(msg->vars[i].offsets); */
     /* } */
     /* free(msg); */
@@ -327,7 +327,7 @@ op_free(void* eventData, void* clientData)
 {
     fp_write_log("OP", "freeing an op message\n");
     op_msg* op = (op_msg*) eventData;
-    if(op->file_name) {
+    if (op->file_name) {
         free(op->file_name);
     }
     free(op);
@@ -337,12 +337,12 @@ op_free(void* eventData, void* clientData)
 int 
 queue_count(FlexpathQueueNode** queue) 
 {
-    if(*queue==NULL) {
+    if (*queue==NULL) {
         return 0;
     }
     int count = 1;
     FlexpathQueueNode* current = *queue;
-    while(current && current->next) {
+    while (current && current->next) {
         count++;
         current = current->next;
     }
@@ -360,8 +360,8 @@ threaded_enqueue(
     int max_size) 
 {
     pthread_mutex_lock(mutex);
-    if(max_size > 0){
-	while(queue_count(queue) > max_size){
+    if (max_size > 0) {
+	while (queue_count(queue) > max_size) {
 	    pthread_cond_wait(condition, mutex);
 	}
     }
@@ -383,23 +383,23 @@ threaded_dequeue(
     int signal_dequeue) 
 {
     pthread_mutex_lock(mutex);
-    while(queue_count(queue) == 0) {
+    while (queue_count(queue) == 0) {
         pthread_cond_wait(condition, mutex);
     }
     FlexpathQueueNode *tail;
     FlexpathQueueNode *prev = NULL;
     tail = *queue;
-    while(tail && tail->next) {
+    while (tail && tail->next) {
         prev=tail;
         tail=tail->next;
     }
-    if(prev) {
+    if (prev) {
         prev->next = NULL;
     } else {
         *queue = NULL;
     }
     pthread_mutex_unlock(mutex);
-    if(signal_dequeue==1) {
+    if (signal_dequeue==1) {
         pthread_cond_broadcast(condition);
     }
     return tail;
@@ -413,12 +413,12 @@ threaded_peek(FlexpathQueueNode** queue,
 {
     pthread_mutex_lock(mutex);
     int q = queue_count(queue);
-    if(q == 0) {	
+    if (q == 0) {	
 	pthread_cond_wait(condition, mutex);
     }
     FlexpathQueueNode* tail;
     tail = *queue;
-    while(tail && tail->next) {
+    while (tail && tail->next) {
         tail=tail->next;
     }
     pthread_mutex_unlock(mutex);
@@ -429,7 +429,7 @@ threaded_peek(FlexpathQueueNode** queue,
 FlexpathVarNode* 
 add_var(FlexpathVarNode* queue, char* varName, FlexpathVarNode* dims, int rank)
 {
-    if(queue) {
+    if (queue) {
         queue->next=add_var(queue->next, varName, dims, rank);
         return queue;
     } else {
@@ -446,7 +446,7 @@ add_var(FlexpathVarNode* queue, char* varName, FlexpathVarNode* dims, int rank)
 void 
 free_vars(FlexpathVarNode* queue)
 {
-    if(queue) {
+    if (queue) {
         free_vars(queue->next);
         free(queue->varName);
         free(queue);
@@ -458,14 +458,14 @@ FlexpathVarNode*
 queue_contains(FlexpathVarNode* queue, const char* name, int rank) 
 {
     int compare_rank = 0;
-    if(rank >= 0 ) {
+    if (rank >= 0 ) {
         compare_rank = 1;
     }
     FlexpathVarNode* tmp = queue;
-    while(tmp) {
-        if(strcmp(tmp->varName, name)==0){
-            if(compare_rank) {
-                if(tmp->rank == rank) {
+    while (tmp) {
+        if (strcmp(tmp->varName, name)==0) {
+            if (compare_rank) {
+                if (tmp->rank == rank) {
                     return tmp;
                 }
             } else {
@@ -534,21 +534,22 @@ get_var_offsets(struct adios_var_struct *v,
 		      uint64_t **local_dimensions,
 		      uint64_t **global_dimensions)
 {
-    struct adios_dimension_struct * dim_list = v->dimensions;	    
-
+    struct adios_dimension_struct * dim_list = v->dimensions;
     int ndims = 0;
     while (dim_list) {
         ndims++;
         dim_list = dim_list->next;
     }
-    dim_list = v->dimensions;	    
-
-    if(ndims){		
+    dim_list = v->dimensions;
+    
+    if (ndims) {
+	uint64_t global = adios_get_dim_value(&dim_list->global_dimension);
+	if (global == 0) { return 0;}
         uint64_t *local_offsets = (uint64_t*)malloc(sizeof(uint64_t) * ndims);
         uint64_t *local_sizes = (uint64_t*)malloc(sizeof(uint64_t) * ndims);
 	uint64_t *global_sizes = (uint64_t*)malloc(sizeof(uint64_t) * ndims);
         int n = 0; 
-        while(dim_list) {		
+        while (dim_list) {
             local_sizes[n] = (uint64_t)adios_get_dim_value(&dim_list->dimension);
             local_offsets[n] = (uint64_t)adios_get_dim_value(&dim_list->local_offset);
 	    global_sizes[n] = (uint64_t)adios_get_dim_value(&dim_list->global_dimension);
@@ -572,40 +573,40 @@ char *multiqueue_action = "{\n\
     int flush_data_count = 0; \n\
     int my_rank = -1;\n\
     attr_list mine;\n\
-    if(EVcount_varMsg()>0) {\n\
+    if (EVcount_varMsg()>0) {\n\
         EVdiscard_and_submit_varMsg(0, 0);\n\
     }\n\
-    if(EVcount_update_step_msg() > 1) {\n\
+    if (EVcount_update_step_msg() > 1) {\n\
         EVdiscard_update_step_msg(0);\n\
     }\n\
-    if(EVcount_drop_evgroup_msg()>0) {\n\
-       if(EVcount_evgroup()>0) {\n\
+    if (EVcount_drop_evgroup_msg()>0) {\n\
+       if (EVcount_evgroup()>0) {\n\
           EVdiscard_evgroup(0);\n\
        }\n\
        EVdiscard_and_submit_drop_evgroup_msg(0,0);\n\
     }\n\
-    if(EVcount_op_msg()>0) {\n\
+    if (EVcount_op_msg()>0) {\n\
         op_msg *msg = EVdata_op_msg(0);\n\
         mine = EVget_attrs_op_msg(0);\n\
         found = attr_ivalue(mine, \"fp_dst_rank\");\n\
-        if(found > 0) {\n\
+        if (found > 0) {\n\
             EVdiscard_and_submit_op_msg(found, 0);\n\
         } else {\n\
             EVdiscard_and_submit_op_msg(0,0);\n\
         }\n\
     }\n\
-    if(EVcount_flush()>0) {\n\
+    if (EVcount_flush()>0) {\n\
         flush *c = EVdata_flush(0);\n\
-         if(c->type == 2) { \n\
-             if(EVcount_evgroup()>0){\n\
+         if (c->type == 2) { \n\
+             if (EVcount_evgroup()>0) {\n\
                evgroup *g = EVdata_evgroup(0); \n\
                g->condition = c->condition;\n\
                EVsubmit(c->process_id+1, g);\n\
                EVdiscard_flush(0);\n\
              }\n\
          }\n\
-         else if(c->type == 3) {\n\
-            if(EVcount_update_step_msg()>0) {\n\
+         else if (c->type == 3) {\n\
+            if (EVcount_update_step_msg()>0) {\n\
                update_step_msg *stepmsg = EVdata_update_step_msg(0);\n\
                stepmsg->condition = c->condition;\n\
                EVsubmit(c->process_id+1, stepmsg);\n\
@@ -617,7 +618,7 @@ char *multiqueue_action = "{\n\
             flush_data_count++;\n\
          }\n\
     }\n\
-    if(EVcount_anonymous()>0){\n\
+    if (EVcount_anonymous()>0) {\n\
         mine = EVget_attrs_anonymous(0);\n\
         found = attr_ivalue(mine, \"fp_dst_rank\");\n\
         EVdiscard_and_submit_anonymous(found+1,0);\n\
@@ -631,7 +632,7 @@ set_field(int type, FMFieldList* field_list_ptr, int fieldNo, int* size)
     FMFieldList field_list = *field_list_ptr;
     switch (type) {
     case adios_unknown:
-	perr("set_field: Bad Type Error\n");
+	fprintf(stderr, "set_field: Bad Type Error %d\n", type);
 	break;
 
     case adios_unsigned_integer:
@@ -640,13 +641,7 @@ set_field(int type, FMFieldList* field_list_ptr, int fieldNo, int* size)
 	field_list[fieldNo].field_offset = *size;
 	*size += sizeof(unsigned int);
 	break;
-
-    case adios_unsigned_long:
-	field_list[fieldNo].field_type = strdup("unsigned long");
-	field_list[fieldNo].field_size = sizeof(unsigned long);
-	field_list[fieldNo].field_offset = *size;
-	*size += sizeof(unsigned long);
-
+	
     case adios_integer:
 	field_list[fieldNo].field_type = strdup("integer");
 	field_list[fieldNo].field_size = sizeof(int);
@@ -654,7 +649,7 @@ set_field(int type, FMFieldList* field_list_ptr, int fieldNo, int* size)
 	*size += sizeof(int);
 	break;
 
-    case adios_real:
+    case adios_real:       
 	field_list[fieldNo].field_type = strdup("float");
 	field_list[fieldNo].field_size = sizeof(float);
 	field_list[fieldNo].field_offset = *size;
@@ -669,12 +664,19 @@ set_field(int type, FMFieldList* field_list_ptr, int fieldNo, int* size)
 	break;
 
     case adios_double:
-	field_list[fieldNo].field_type = strdup("float");
+	field_list[fieldNo].field_type = strdup("double");
 	field_list[fieldNo].field_size = sizeof(double);
 	field_list[fieldNo].field_offset = *size;
 	*size += sizeof(double);
 	break;
 
+    case adios_long_double:
+	field_list[fieldNo].field_type = strdup("double");
+	field_list[fieldNo].field_size = sizeof(long double);
+	field_list[fieldNo].field_offset = *size;
+	*size += sizeof(long double);
+	break;
+
     case adios_byte:
 	field_list[fieldNo].field_type = strdup("char");
 	field_list[fieldNo].field_size = sizeof(char);
@@ -682,6 +684,20 @@ set_field(int type, FMFieldList* field_list_ptr, int fieldNo, int* size)
 	*size += sizeof(char);
 	break;
 
+    case adios_long:
+	field_list[fieldNo].field_type = strdup("integer");
+	field_list[fieldNo].field_size = sizeof(long);
+	field_list[fieldNo].field_offset = *size;
+	*size += sizeof(long);
+	break;
+	
+    case adios_unsigned_long:
+	field_list[fieldNo].field_type = strdup("unsigned integer");
+	field_list[fieldNo].field_size = sizeof(unsigned long);
+	field_list[fieldNo].field_offset = *size;
+	*size += sizeof(unsigned long);
+	break;
+
     case adios_complex:
         field_list[fieldNo].field_type = strdup("complex");
         field_list[fieldNo].field_size = sizeof(complex_dummy);
@@ -697,7 +713,7 @@ set_field(int type, FMFieldList* field_list_ptr, int fieldNo, int* size)
         break;
 
     default:
-	perr("set_field: Unknown Type Error\n");
+	fprintf(stderr, "set_field: Unknown Type Error %d\n", type);
 	break;
     }
     *field_list_ptr = field_list;
@@ -718,7 +734,7 @@ internal_find_field(char *name, FMFieldList flist)
 void 
 mem_check(void* ptr, const char* str) 
 {
-    if(!ptr) {
+    if (!ptr) {
         adios_error(err_no_memory, "Cannot allocate memory for flexpath %s.", str);
     }
 }
@@ -776,17 +792,16 @@ set_format(struct adios_group_struct *t,
 
 	// use the mangled name for the field.
 	field_list[fieldNo].field_name = fullname;
-        if(fullname!=NULL) {
+        if (fullname!=NULL) {
             int num_dims = 0;
             char atom_name[200] = "";
             FlexpathVarNode *dims=NULL;
-            if(adios_var->dimensions) {
+            if (adios_var->dimensions) {
                 struct adios_dimension_struct *adim = adios_var->dimensions;  
 		
                 // attach appropriate attrs for dimensions	
-                for(; adim != NULL; adim = adim->next) {
+                for (; adim != NULL; adim = adim->next) {
                     num_dims++;		    
-                    
 		    // have to change get_alt_name to append FPVAR at the start of each varname.
                     char *vname = get_dim_name(&adim->dimension);
                     if (vname) {
@@ -798,13 +813,16 @@ set_format(struct adios_group_struct *t,
 			set_attr_dimensions(fullname, aname, num_dims, fileData->attrs);
 		    }
                     char *gname = get_dim_name(&adim->global_dimension);
-		    if(gname) {
+		    if (gname) {
 			fileData->globalCount++;
 			//char *name = find_fixed_name(currentFm, gname);
 			char *aname = get_alt_name(fullname, gname);
 			dims=add_var(dims, strdup(aname), NULL, 0);
 			set_attr_dimensions(fullname, aname, num_dims, fileData->attrs);
 		    }
+		    if (adim->global_dimension.rank > 0) {
+			fileData->globalCount++;
+		    }
                 }
             }
             // attach ndims attr
@@ -846,7 +864,7 @@ set_format(struct adios_group_struct *t,
 	    }
 	    v_offset *= -1;
 		  
-	    while(currentFm->size % 8 != 0) {
+	    while (currentFm->size % 8 != 0) {
 		currentFm->size ++;					
 	    }
 		  
@@ -884,21 +902,6 @@ set_format(struct adios_group_struct *t,
 		{  currentFm->size += (v_offset * sizeof(unsigned int));  } 
 		break;
 
-	    case adios_unsigned_long: // needs to be unsigned integer in ffs
-                // to distinguish on reader_side, I have to look at the size also
-		field_list[fieldNo].field_type =
-		    (char *) malloc(sizeof(char) * 255);
-		snprintf((char *) field_list[fieldNo].field_type, 255,
-			 "unsigned long%s", dims);
-		field_list[fieldNo].field_size = sizeof(unsigned long);
-		      
-		field_list[fieldNo].field_offset = currentFm->size;
-		if (v_offset == 0 ) // pointer to variably sized array
-		{ currentFm->size += sizeof(void *);  } 
-		else // statically sized array allocated inline
-		{  currentFm->size += (v_offset * sizeof(unsigned long));  } 
-		break;
-
 	    case adios_real:
 		field_list[fieldNo].field_type =
 		    (char *) malloc(sizeof(char) * 255);
@@ -931,6 +934,19 @@ set_format(struct adios_group_struct *t,
 		else // statically sized array allocated inline
 		{  currentFm->size += (v_offset * sizeof(double));  } 
 		break;
+		
+	    case adios_long_double:
+		field_list[fieldNo].field_type =
+		    (char *) malloc(sizeof(char) * 255);
+		snprintf((char *) field_list[fieldNo].field_type, 255,
+			 "double%s", dims);
+		field_list[fieldNo].field_size = sizeof(long double);
+		field_list[fieldNo].field_offset = currentFm->size;
+		if (v_offset == 0 ) // pointer to variably sized array
+		{ currentFm->size += sizeof(void *);  } 
+		else // statically sized array allocated inline
+		{  currentFm->size += (v_offset * sizeof(long double));  } 
+		break;
 
 	    case adios_byte:
 		field_list[fieldNo].field_type =
@@ -945,7 +961,37 @@ set_format(struct adios_group_struct *t,
 		{  currentFm->size += (v_offset * sizeof(char));  } 
 		break;
 
-            case adios_complex:
+	    case adios_long: // needs to be unsigned integer in ffs
+                // to distinguish on reader_side, I have to look at the size also
+		field_list[fieldNo].field_type =
+		    (char *) malloc(sizeof(char) * 255);
+		snprintf((char *) field_list[fieldNo].field_type, 255,
+			 "integer%s", dims);
+		field_list[fieldNo].field_size = sizeof(long);
+		      
+		field_list[fieldNo].field_offset = currentFm->size;
+		if (v_offset == 0 ) // pointer to variably sized array
+		{ currentFm->size += sizeof(void *);  } 
+		else // statically sized array allocated inline
+		{  currentFm->size += (v_offset * sizeof(long));  } 
+		break;
+		
+	    case adios_unsigned_long: // needs to be unsigned integer in ffs
+                // to distinguish on reader_side, I have to look at the size also
+		field_list[fieldNo].field_type =
+		    (char *) malloc(sizeof(char) * 255);
+		snprintf((char *) field_list[fieldNo].field_type, 255,
+			 "unsigned integer%s", dims);
+		field_list[fieldNo].field_size = sizeof(unsigned long);
+		      
+		field_list[fieldNo].field_offset = currentFm->size;
+		if (v_offset == 0 ) // pointer to variably sized array
+		{ currentFm->size += sizeof(void *);  } 
+		else // statically sized array allocated inline
+		{  currentFm->size += (v_offset * sizeof(unsigned long));  } 
+		break;
+
+	    case adios_complex:
 		field_list[fieldNo].field_type =
 		    (char *) malloc(sizeof(char) * 255);
 		snprintf((char *) field_list[fieldNo].field_type, 255, "complex%s",
@@ -1013,7 +1059,7 @@ set_format(struct adios_group_struct *t,
     format[1].field_list = complex_dummy_field_list;
     format[2].format_name = strdup("double_complex");
     format[2].field_list = double_complex_dummy_field_list;
-    
+
     format[0].struct_size = currentFm->size;
     format[1].struct_size = sizeof(complex_dummy);
     format[2].struct_size = sizeof(double_complex_dummy);
@@ -1031,20 +1077,20 @@ void* copy_buffer(void* buffer, int rank, FlexpathWriteFileData* fileData)
     while (f->field_name != NULL)
     {
         FlexpathVarNode* a;
-        if(!queue_contains(fileData->askedVars, f->field_name, rank)) {
-            if((a=queue_contains(fileData->formatVars, f->field_name, -1)) 
+        if (!queue_contains(fileData->askedVars, f->field_name, rank)) {
+            if ((a=queue_contains(fileData->formatVars, f->field_name, -1)) 
 	       && 
 	       (a->dimensions != NULL)) {
                 FlexpathVarNode* dim = a->dimensions;
-                while(dim) {
+                while (dim) {
                     FMField *f2 = fileData->fm->format[0].field_list;
-                    while(f2->field_name != NULL) {
-                        if(strcmp(f2->field_name, dim->varName)==0) {
+                    while (f2->field_name != NULL) {
+                        if (strcmp(f2->field_name, dim->varName)==0) {
                             break;
                         }
                         f2++;
                     }
-                    if(f2->field_name != NULL) {
+                    if (f2->field_name != NULL) {
                         memset(&temp[f2->field_offset], 0, f2->field_size);
                     }
                     dim = dim->next;
@@ -1067,7 +1113,7 @@ process_data_flush(FlexpathWriteFileData *fileData,
     fileData->attrs = set_dst_rank_atom(fileData->attrs, flushMsg->process_id);
     fileData->attrs = set_flush_id_atom(fileData->attrs, flushMsg->id);
     
-    if(!fileData->bridges[flushMsg->process_id].opened) {
+    if (!fileData->bridges[flushMsg->process_id].opened) {
 	fileData->bridges[flushMsg->process_id].opened=1;
 	fileData->openCount++;
     }
@@ -1089,7 +1135,7 @@ process_open_msg(FlexpathWriteFileData *fileData, op_msg *open)
 {
     fileData->bridges[open->process_id].step = open->step;
     fileData->bridges[open->process_id].condition = open->condition;
-    if(!fileData->bridges[open->process_id].created){
+    if (!fileData->bridges[open->process_id].created) {
 	fileData->bridges[open->process_id].created = 1;
 	fileData->bridges[open->process_id].myNum = 
 	    EVcreate_bridge_action(
@@ -1104,7 +1150,7 @@ process_open_msg(FlexpathWriteFileData *fileData, op_msg *open)
 			    fileData->bridges[open->process_id].myNum);
     }	
 	
-    if (open->step == fileData->readerStep){
+    if (open->step == fileData->readerStep) {
 	pthread_mutex_lock(&fileData->openMutex);
 	fileData->openCount++;  
 	fileData->bridges[open->process_id].opened = 1;
@@ -1119,7 +1165,7 @@ process_open_msg(FlexpathWriteFileData *fileData, op_msg *open)
 	fileData->attrs = set_dst_rank_atom(fileData->attrs, open->process_id+1);
 	EVsubmit_general(fileData->opSource, ack, op_free, fileData->attrs);
     } 
-    else if(open->step < fileData->readerStep) {
+    else if (open->step < fileData->readerStep) {
 	log_error("Flexpath method control_thread: Received Past Step Open\n");
     } 
     else {
@@ -1137,7 +1183,7 @@ process_close_msg(FlexpathWriteFileData *fileData, op_msg *close)
     fileData->bridges[close->process_id].condition = close->condition;
     pthread_mutex_unlock(&fileData->openMutex);
 
-    if(fileData->openCount==0) {
+    if (fileData->openCount==0) {
 	FlexpathQueueNode* node = threaded_dequeue(&fileData->dataQueue, 
 						   &fileData->dataMutex, 
 						   &fileData->dataCondition, 1);
@@ -1194,7 +1240,7 @@ flush_handler(CManager cm, void* vevent, void* client_data, attr_list attrs)
 }
 
 static int
-drop_evgroup_handler(CManager cm, void *vevent, void *client_data, attr_list attrs){
+drop_evgroup_handler(CManager cm, void *vevent, void *client_data, attr_list attrs) {
     drop_evgroup_msg *msg = vevent;
     // will have to change when not using control thread.
     CMCondition_signal(cm, msg->condition);    
@@ -1207,10 +1253,10 @@ op_handler(CManager cm, void* vevent, void* client_data, attr_list attrs)
     FlexpathWriteFileData* fileData = (FlexpathWriteFileData*) client_data;
     op_msg* msg = (op_msg*) vevent;
     EVtake_event_buffer(cm, vevent);
-    if(msg->type == OPEN_MSG) {
+    if (msg->type == OPEN_MSG) {
         threaded_enqueue(&fileData->controlQueue, msg, OPEN, 
 			 &fileData->controlMutex, &fileData->controlCondition, -1);
-    } else if(msg->type == CLOSE_MSG) {
+    } else if (msg->type == CLOSE_MSG) {
         threaded_enqueue(&fileData->controlQueue, msg, CLOSE, 
 			 &fileData->controlMutex, &fileData->controlCondition, -1);  			
     }
@@ -1225,15 +1271,15 @@ control_thread(void *arg)
     int rank = fileData->rank;
     FlexpathQueueNode *controlMsg;
     FlexpathQueueNode *dataNode;
-    while(1) {
-	if((controlMsg = threaded_dequeue(&fileData->controlQueue, 
+    while (1) {
+	if ((controlMsg = threaded_dequeue(&fileData->controlQueue, 
 	    &fileData->controlMutex, &fileData->controlCondition, 0))) {
-	    if(controlMsg->type==VAR){
+	    if (controlMsg->type==VAR) {
 		Var_msg *varMsg = (Var_msg*) controlMsg->data;
 		process_var_msg(fileData, varMsg);
 		EVreturn_event_buffer(flexpathWriteData.cm,controlMsg->data);
 	    }
-	    else if(controlMsg->type==DATA_FLUSH) {
+	    else if (controlMsg->type==DATA_FLUSH) {
 		Flush_msg *flushMsg = (Flush_msg*)controlMsg->data;
 		dataNode = threaded_peek(&fileData->dataQueue, 
 					 &fileData->dataMutex, 
@@ -1241,17 +1287,17 @@ control_thread(void *arg)
 		process_data_flush(fileData, flushMsg, dataNode);
 
 	    }
-	    else if(controlMsg->type==OPEN) {
+	    else if (controlMsg->type==OPEN) {
                 op_msg *open = (op_msg*) controlMsg->data;
 		process_open_msg(fileData, open);                
 		EVreturn_event_buffer(flexpathWriteData.cm, open);
             }
-	    else if(controlMsg->type==CLOSE) {
+	    else if (controlMsg->type==CLOSE) {
                 op_msg* close = (op_msg*) controlMsg->data;
 		process_close_msg(fileData, close);
 		EVreturn_event_buffer(flexpathWriteData.cm, close);
 	    }
-	    else{
+	    else {
 		log_error("control_thread: Unrecognized Control Message\n");
 	    }
 	}
@@ -1264,10 +1310,10 @@ void
 add_open_file(FlexpathWriteFileData* newFile) 
 {
     FlexpathWriteFileData* last = flexpathWriteData.openFiles;
-    while(last && last->next) {
+    while (last && last->next) {
         last = last->next;
     }
-    if(last) {
+    if (last) {
         last->next = newFile;
     } else {
         flexpathWriteData.openFiles = newFile;
@@ -1279,7 +1325,7 @@ FlexpathWriteFileData*
 find_open_file(char* name) 
 {
     FlexpathWriteFileData* file = flexpathWriteData.openFiles;
-    while(file && strcmp(file->name, name)) {
+    while (file && strcmp(file->name, name)) {
         file = file->next;
     }
     return file;
@@ -1299,8 +1345,8 @@ adios_flexpath_init(const PairStruct *params, struct adios_method_struct *method
     flexpathWriteData.cm = CManager_create();
     atom_t CM_TRANSPORT = attr_atom_from_string("CM_TRANSPORT");
     char * transport = getenv("CMTransport");
-    if(transport == NULL){
-	if(CMlisten(flexpathWriteData.cm) == 0) {
+    if (transport == NULL) {
+	if (CMlisten(flexpathWriteData.cm) == 0) {
 	    fprintf(stderr, "error: unable to initialize connection manager.\n");
 	    exit(1);
 	}
@@ -1316,7 +1362,7 @@ adios_flexpath_init(const PairStruct *params, struct adios_method_struct *method
     
     // fork communications thread
     int forked = CMfork_comm_thread(flexpathWriteData.cm);   
-    if(!forked) {
+    if (!forked) {
 	fprintf(stderr, "Wrtier error forking comm thread\n");
     }
 }
@@ -1326,13 +1372,13 @@ adios_flexpath_open(struct adios_file_struct *fd,
 		    struct adios_method_struct *method, 
 		    MPI_Comm comm) 
 {    
-    if( fd == NULL || method == NULL) {
+    if ( fd == NULL || method == NULL) {
         perr("open: Bad input parameters\n");
         return -1;
     }
 
     // file creation
-    if(find_open_file(method->group->name)) {
+    if (find_open_file(method->group->name)) {
         // stream already open
         return 0;
     }
@@ -1343,7 +1389,7 @@ adios_flexpath_open(struct adios_file_struct *fd,
     
     fileData->maxQueueSize = 1;
     fileData->use_ctrl_thread = 1;
-    if(method->parameters) {
+    if (method->parameters) {
         sscanf(method->parameters,"QUEUE_SIZE=%d;", &fileData->maxQueueSize);
     }
    
@@ -1376,7 +1422,7 @@ adios_flexpath_open(struct adios_file_struct *fd,
     MPI_Comm_size((fileData->mpiComm), &fileData->size);
     char *recv_buff = NULL;
     char sendmsg[CONTACT_LENGTH];
-    if(fileData->rank == 0) {
+    if (fileData->rank == 0) {
         recv_buff = (char *) malloc(fileData->size*CONTACT_LENGTH*sizeof(char));
     }
         
@@ -1389,10 +1435,10 @@ adios_flexpath_open(struct adios_file_struct *fd,
         CONTACT_LENGTH, MPI_CHAR, 0, (fileData->mpiComm));
 
     // rank 0 prints contact info to file
-    if(fileData->rank == 0) {
+    if (fileData->rank == 0) {
         sprintf(writer_info_filename, "%s_%s", fd->name, "writer_info.txt");
         FILE* writer_info = fopen(writer_info_filename,"w");
-        for(i=0; i<fileData->size; i++) {
+        for (i=0; i<fileData->size; i++) {
             fprintf(writer_info, "%s\n",&recv_buff[i*CONTACT_LENGTH]); 
         }
         fclose(writer_info);
@@ -1401,7 +1447,7 @@ adios_flexpath_open(struct adios_file_struct *fd,
     // poll file - race condition issues
     FILE* reader_ready = NULL;
     sprintf(reader_ready_filename, "%s_%s", fd->name, "reader_ready.txt");
-    while(!reader_ready){
+    while (!reader_ready) {
 	reader_ready = fopen(reader_ready_filename,"r");
     }
     fclose(reader_ready);
@@ -1409,14 +1455,14 @@ adios_flexpath_open(struct adios_file_struct *fd,
     // read contact list
     sprintf(reader_info_filename, "%s_%s", fd->name, "reader_info.txt");
     FILE* reader_info = fopen(reader_info_filename, "r");
-    if(!reader_info){
+    if (!reader_info) {
 	reader_info = fopen(reader_info_filename, "r");
     }
     char in_contact[CONTACT_LENGTH] = "";
     int numBridges = 0;
     int stone_num;
     // build a bridge per line
-    while(fscanf(reader_info, "%d:%s",&stone_num, in_contact)!=EOF){
+    while (fscanf(reader_info, "%d:%s",&stone_num, in_contact)!=EOF) {
 	//fprintf(stderr, "reader contact: %d:%s\n", stone_num, in_contact);
         fileData->bridges = realloc(fileData->bridges, sizeof(FlexpathStone) * (numBridges + 1));
         attr_list contact_list = attr_list_from_string(in_contact);
@@ -1433,7 +1479,7 @@ adios_flexpath_open(struct adios_file_struct *fd,
     MPI_Barrier((fileData->mpiComm));
     
     // cleanup of reader files (writer is done with it).
-    if(fileData->rank == 0){
+    if (fileData->rank == 0) {
 	unlink(reader_info_filename);
 	unlink(reader_ready_filename);
     }
@@ -1441,14 +1487,14 @@ adios_flexpath_open(struct adios_file_struct *fd,
 	
     //process group format
     struct adios_group_struct *t = method->group;
-    if(t == NULL){
+    if (t == NULL) {
 	adios_error(err_invalid_group, "Invalid group.\n");
 	return err_invalid_group;
     }
     fileData->host_language = t->adios_host_language_fortran;
     struct adios_var_struct *fields = t->vars;
 	
-    if(fields == NULL){
+    if (fields == NULL) {
 	adios_error(err_invalid_group, "Group has no variables.\n");
 	return err_invalid_group;
     }	
@@ -1514,7 +1560,7 @@ adios_flexpath_open(struct adios_file_struct *fd,
     fileData->fm->ioFormat = register_data_format(my_context, fileData->fm->format);
     
     sprintf(writer_ready_filename, "%s_%s", fd->name, "writer_ready.txt");
-    if(fileData->rank == 0) {
+    if (fileData->rank == 0) {
         FILE* writer_info = fopen(writer_ready_filename, "w");
         fprintf(writer_info, "ready");
         fclose(writer_info);
@@ -1532,7 +1578,7 @@ extern void
 adios_flexpath_write(
     struct adios_file_struct *fd, 
     struct adios_var_struct *f, 
-    void *data, 
+    const void *data, 
     struct adios_method_struct *method) 
 {
     FlexpathWriteFileData* fileData = find_open_file(method->group->name);
@@ -1555,10 +1601,15 @@ adios_flexpath_write(
 	if (!f->dimensions) {
 	    if (data) {
 		//why wouldn't it have data?
-		memcpy(&fm->buffer[field->field_offset], data, field->field_size);                
+		if (f->type == adios_string) {
+		    char *tmpstr = strdup((char*)data);
+		    set_FMPtrField_by_name(flist, fullname, fm->buffer, tmpstr);
+		} else {
+		    memcpy(&fm->buffer[field->field_offset], data, field->field_size);
+		}
 
 		//scalar quantities can have FlexpathAltNames also so assign those
-		if(field->field_name != NULL) {
+		if (field->field_name != NULL) {
 					
 		    FlexpathDimNames *d = NULL;
 		    for (d = fm->dimList.lh_first; d != NULL; d = d->entries.le_next) {
@@ -1567,11 +1618,15 @@ adios_flexpath_write(
 			    //check if there are FlexpathAltNames
 			    FlexpathAltName *a = NULL;
 			    for (a = d->altList.lh_first; a != NULL; a = a->entries.le_next) {
-				/* fprintf(stderr, "ALTNAME: %s, DIM: %s FIELD: %s\n",  */
-				/* 	a->name, d->name, field->field_name); */
-				memcpy(&fm->buffer[a->field->field_offset], 
-				       data, 
-				       a->field->field_size);
+				if (f->type == adios_string) {
+				    char *tmpstr = strdup((char*)data);
+				    set_FMPtrField_by_name(flist, fullname, fm->buffer, tmpstr);
+				    //(strcpy(&fm->buffer[a->field->field_offset], (char*)data));
+				} else {
+				    memcpy(&fm->buffer[a->field->field_offset], 
+					   data, 
+					   a->field->field_size);
+				}
 			    }
 			}
 		    }
@@ -1608,8 +1663,8 @@ adios_flexpath_write(
 extern void 
 adios_flexpath_close(struct adios_file_struct *fd, struct adios_method_struct *method) 
 {
-    FlexpathWriteFileData* fileData = find_open_file(method->group->name);
-    void* buffer = malloc(fileData->fm->size);    
+    FlexpathWriteFileData *fileData = find_open_file(method->group->name);
+    void *buffer = malloc(fileData->fm->size);    
     memcpy(buffer, fileData->fm->buffer, fileData->fm->size);
 
     threaded_enqueue(&fileData->dataQueue, buffer, 
@@ -1618,25 +1673,21 @@ adios_flexpath_close(struct adios_file_struct *fd, struct adios_method_struct *m
 		     &fileData->dataCondition,
 		     fileData->maxQueueSize);
     
-    int c = 0;
- 
     // now gather offsets and send them via MPI to root
     struct adios_group_struct * g = fd->group;
     struct adios_var_struct * list = g->vars;
     evgroup *gp = malloc(sizeof(evgroup));    
     gp->group_name = strdup(method->group->name);
     gp->process_id = fileData->rank;
-    if(fileData->globalCount == 0){
+    if (fileData->globalCount == 0 ) {
 
 	gp->num_vars = 0;
 	gp->step = fileData->writerStep;
 	gp->vars = NULL;
-	//fileData->gp = gp;       
-	//fileData->attrs = set_size_atom(fileData->attrs, fileData->size);
 	EVsubmit_general(fileData->offsetSource, gp, evgroup_msg_free, fileData->attrs);
     }
 
-    else{	
+    else {	
         // process local offsets here	
 	int num_gbl_vars = 0;
         global_var * gbl_vars = NULL;
@@ -1644,7 +1695,7 @@ adios_flexpath_close(struct adios_file_struct *fd, struct adios_method_struct *m
 	int myrank = fileData->rank;
 	int commsize = fileData->size;
 
-	while(list){
+	while (list) {
 	    char *fullname = resolve_path_name(list->path, list->name);
 	    //int num_local_offsets = 0;
 	    uint64_t *local_offsets = NULL;
@@ -1661,7 +1712,7 @@ adios_flexpath_close(struct adios_file_struct *fd, struct adios_method_struct *m
                 reverse_dims(global_dimensions, num_local_offsets);
             }
 
-	    if(num_local_offsets > 0){
+	    if (num_local_offsets > 0) {
 		uint64_t *all_offsets = NULL;
 		uint64_t *all_local_dims = NULL;
 		
@@ -1718,7 +1769,7 @@ extern void
 adios_flexpath_finalize(int mype, struct adios_method_struct *method) 
 {
     FlexpathWriteFileData* fileData = flexpathWriteData.openFiles;
-    while(fileData) {
+    while (fileData) {
 
 	update_step_msg *stepmsg = malloc(sizeof(update_step_msg));
 	stepmsg->finalized = 1;
@@ -1727,7 +1778,7 @@ adios_flexpath_finalize(int mype, struct adios_method_struct *method)
 	EVsubmit_general(fileData->stepSource, stepmsg, update_step_msg_free, fileData->attrs);
 
         pthread_mutex_lock(&fileData->dataMutex);
-        while(fileData->dataQueue != NULL) {
+        while (fileData->dataQueue != NULL) {
 	    pthread_cond_wait(&fileData->dataCondition, &fileData->dataMutex);
 	}
 	pthread_mutex_unlock(&fileData->dataMutex);
@@ -1772,21 +1823,21 @@ adios_flexpath_get_write_buffer(struct adios_file_struct *fd,
 {
     uint64_t mem_allowed;
 
-    if (*size == 0){    
+    if (*size == 0) {    
         *buffer = 0;
         return;
     }
 
-    if (v->data && v->free_data == adios_flag_yes){   
+    if (v->adata && v->free_data == adios_flag_yes) {   
         adios_method_buffer_free (v->data_size);
-        free (v->data);
-        v->data = NULL;
+        free (v->adata);
+        v->data = v->adata = NULL;
     }
 
     mem_allowed = adios_method_buffer_alloc (*size);
-    if (mem_allowed == *size){   
+    if (mem_allowed == *size) {   
         *buffer = malloc (*size);
-        if (!*buffer){        
+        if (!*buffer) {        
             adios_method_buffer_free (mem_allowed);
             log_error ("ERROR: Out of memory allocating %llu bytes for %s in %s:%s()\n"
                     ,*size, v->name, __FILE__, __func__
@@ -1798,14 +1849,14 @@ adios_flexpath_get_write_buffer(struct adios_file_struct *fd,
             *size = 0;
             *buffer = 0;
         }
-        else{        
+        else {        
             v->got_buffer = adios_flag_yes;
             v->free_data = adios_flag_yes;
             v->data_size = mem_allowed;
             v->data = *buffer;
         }
     }
-    else{    
+    else {    
         adios_method_buffer_free (mem_allowed);
         log_error ("OVERFLOW: Cannot allocate requested buffer of %llu "
                          "bytes for %s in %s:%s()\n"
