@@ -426,7 +426,7 @@ adios_mpi_lustre_get_striping_unit(MPI_File fh, char *filename)
 static uint64_t
 adios_mpi_lustre_striping_unit_write(MPI_File    fh,
                               MPI_Offset  offset,
-                              void        *buf,
+                              const void *buf,
                               uint64_t    len,
                               uint64_t    block_unit 
                               )
@@ -444,7 +444,7 @@ adios_mpi_lustre_striping_unit_write(MPI_File    fh,
     if (block_unit > 0) {
         MPI_Offset  rem_off = offset;
         uint64_t    rem_size = len;
-        char       *buf_ptr = buf;
+        const char       *buf_ptr = buf;
 
         err = 0;
         while (rem_size > 0) {
@@ -455,7 +455,7 @@ adios_mpi_lustre_striping_unit_write(MPI_File    fh,
 #ifdef _WKL_CHECK_STRIPE_IO
 printf("adios_mpi_lustre_striping_unit_write offset=%12lld len=%12d\n",offset,write_len);offset+=write_len;
 #endif
-            MPI_File_write (fh, buf_ptr, write_len, MPI_BYTE, &status);
+            MPI_File_write (fh, (char*)buf_ptr, write_len, MPI_BYTE, &status);
             MPI_Get_count(&status, MPI_BYTE, &ret_len);
             if (ret_len < 0) {err = ret_len; break;}
             err += ret_len;
@@ -470,11 +470,11 @@ printf("adios_mpi_lustre_striping_unit_write offset=%12lld len=%12d\n",offset,wr
         uint64_t to_write = len;
         int write_len = 0;
         int count;
-        char * buf_ptr = buf;
+        const char * buf_ptr = buf;
         while (total_written < len)
         {
             write_len = (to_write > MAX_MPIWRITE_SIZE) ? MAX_MPIWRITE_SIZE : to_write;
-            MPI_File_write (fh, buf_ptr, write_len, MPI_BYTE, &status);
+            MPI_File_write (fh, (char*)buf_ptr, write_len, MPI_BYTE, &status);
             MPI_Get_count(&status, MPI_BYTE, &count);
             if (count != write_len)
             {
@@ -695,7 +695,7 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
                 MPI_File_read (md->fh, md->b.buff, md->b.pg_size, MPI_BYTE
                               ,&md->status
                               );
-                adios_parse_process_group_index_v1 (&md->b, &md->index->pg_root);
+                adios_parse_process_group_index_v1 (&md->b, &md->index->pg_root, &md->index->pg_tail);
 
 #if 1
                 adios_init_buffer_read_vars_index (&md->b);
@@ -1026,7 +1026,25 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
                     MPI_File_read (md->fh, md->b.buff, md->b.pg_size, MPI_BYTE
                                   ,&md->status
                                   );
-                    adios_parse_process_group_index_v1 (&md->b ,&md->index->pg_root);
+                    adios_parse_process_group_index_v1 (&md->b ,&md->index->pg_root, &md->index->pg_tail);
+                    // find the largest time index so we can append properly
+                    struct adios_index_process_group_struct_v1 * p;
+                    uint32_t max_time_index = 0;
+                    p = md->index->pg_root;
+                    while (p)
+                    {
+                        if (p->time_index > max_time_index)
+                            max_time_index = p->time_index;
+                        p = p->next;
+                    }
+                    if (fd->mode == adios_mode_append) {
+                        ++max_time_index;
+                    }
+                    fd->group->time_index = max_time_index;
+                    MPI_Bcast (&fd->group->time_index, 1, MPI_INT, 0
+                              ,md->group_comm
+                              );
+
 
                     adios_init_buffer_read_vars_index (&md->b);
                     MPI_File_seek (md->fh, md->b.vars_index_offset
@@ -1058,6 +1076,9 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
                 {
                     fd->base_offset = 0;
                     fd->pg_start_in_file = 0;
+                    MPI_Bcast (&fd->group->time_index, 1, MPI_INT, 0
+                              ,md->group_comm
+                              );
                 }
 
                 MPI_File_close (&md->fh);
@@ -1300,7 +1321,7 @@ enum ADIOS_FLAG adios_mpi_lustre_should_buffer (struct adios_file_struct * fd
 
 void adios_mpi_lustre_write (struct adios_file_struct * fd
                      ,struct adios_var_struct * v
-                     ,void * data
+                     ,const void * data
                      ,struct adios_method_struct * method
                      )
 {
@@ -1316,7 +1337,7 @@ void adios_mpi_lustre_write (struct adios_file_struct * fd
         {
             if (v->free_data == adios_flag_yes)
             {
-                free (v->data);
+                free (v->adata);
                 adios_method_buffer_free (v->data_size);
             }
         }
@@ -1406,10 +1427,10 @@ void adios_mpi_lustre_get_write_buffer (struct adios_file_struct * fd
         return;
     }
 
-    if (v->data && v->free_data)
+    if (v->adata && v->free_data)
     {
         adios_method_buffer_free (v->data_size);
-        free (v->data);
+        free (v->adata);
     }
 
     mem_allowed = adios_method_buffer_alloc (*size);
@@ -1456,7 +1477,7 @@ void adios_mpi_lustre_read (struct adios_file_struct * fd
                     ,struct adios_method_struct * method
                     )
 {
-    v->data = buffer;
+    v->data = v->adata = buffer;
     v->data_size = buffer_size;
 }
 
@@ -1520,7 +1541,7 @@ static void adios_mpi_lustre_do_read (struct adios_file_struct * fd
 
                 if (v1)
                 {
-                    var_payload.payload = v1->data;
+                    var_payload.payload = v1->adata;
                     adios_parse_var_data_payload_v1 (&md->b, &var_header
                                                     ,&var_payload
                                                     ,v1->data_size
@@ -1588,7 +1609,7 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
             struct adios_var_struct * v = fd->group->vars;
             while (v)
             {
-                v->data = 0;
+                v->data = v->adata = 0;
                 v = v->next;
             }
 
@@ -1756,9 +1777,7 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                         md->b.length = index_sizes [i];
                         md->b.offset = 0;
 
-                        adios_parse_process_group_index_v1 (&md->b
-                                                           ,&new_pg_root
-                                                           );
+                        adios_parse_process_group_index_v1 (&md->b, &new_pg_root, NULL);
                         adios_parse_vars_index_v1 (&md->b, &new_vars_root, NULL, NULL);
                         // do not merge attributes from other processes from 1.4
                         /*
@@ -1767,7 +1786,7 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                                                         );
                          */
                         adios_merge_index_v1 (md->index, new_pg_root, 
-                                              new_vars_root, new_attrs_root);
+                                              new_vars_root, new_attrs_root, 0);
                         new_pg_root = 0;
                         new_vars_root = 0;
                         new_attrs_root = 0;
@@ -2008,9 +2027,7 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                         md->b.length = index_sizes [i];
                         md->b.offset = 0;
 
-                        adios_parse_process_group_index_v1 (&md->b
-                                                           ,&new_pg_root
-                                                           );
+                        adios_parse_process_group_index_v1 (&md->b, &new_pg_root, NULL);
                         adios_parse_vars_index_v1 (&md->b, &new_vars_root, NULL, NULL);
                         // do not merge attributes from other processes from 1.4
                         /*
@@ -2019,7 +2036,7 @@ void adios_mpi_lustre_close (struct adios_file_struct * fd
                                                         );
                          */
                         adios_merge_index_v1 (md->index, new_pg_root, 
-                                              new_vars_root, new_attrs_root);
+                                              new_vars_root, new_attrs_root, 0);
                         new_pg_root = 0;
                         new_vars_root = 0;
                         new_attrs_root = 0;
