@@ -64,6 +64,7 @@ struct adios_POSIX_data_struct
     int rank;
     int size;
 #endif
+    int g_have_mdf;
 };
 
 
@@ -90,6 +91,7 @@ void adios_posix_init (const PairStruct * parameters
     p->rank = 0;
     p->size = 0;
 #endif
+    p->g_have_mdf = 1;
 }
 
 
@@ -114,6 +116,28 @@ int adios_posix_open (struct adios_file_struct * fd
     struct adios_POSIX_data_struct * p = (struct adios_POSIX_data_struct *)
                                                           method->method_data;
 
+    char *temp_string, *m_size;
+
+    temp_string = (char *) malloc (strlen (method->parameters) + 1);
+    strcpy (temp_string, method->parameters);
+    trim_spaces (temp_string);
+
+    if ( (m_size = strstr (temp_string, "have_metadata_file")) )
+    {
+        char * m = strchr (m_size, '=');
+        char * n = strtok (m, ";");
+
+        if (!n)
+            p->g_have_mdf = atoi (n + 1);
+        else
+            p->g_have_mdf = atoi (m + 1);
+    }
+    else
+    {
+        // by default, write metadata file. 
+        p->g_have_mdf = 1;
+    }
+
 #if defined ADIOS_TIMERS || defined ADIOS_TIMER_EVENTS
     int timer_count = 7;
     char ** timer_names = (char**) malloc (timer_count * sizeof (char*) );
@@ -276,7 +300,7 @@ START_TIMER (ADIOS_TIMER_POSIX_AD_OPEN);
 
 #ifdef HAVE_MPI
             // open metadata file
-            if (p->group_comm != MPI_COMM_SELF)
+            if (p->group_comm != MPI_COMM_SELF && p->g_have_mdf)
             {
                 if (p->rank == 0)
                 {
@@ -397,12 +421,13 @@ START_TIMER (ADIOS_TIMER_POSIX_AD_OPEN);
                 {
                     case 1:
                     case 2:
+                    case 3:
                         // read the old stuff and set the base offset
                         adios_posix_read_index_offsets (&p->b);
                         adios_parse_index_offsets_v1 (&p->b);
 
                         adios_posix_read_process_group_index (&p->b);
-                        adios_parse_process_group_index_v1 (&p->b, &p->index->pg_root);
+                        adios_parse_process_group_index_v1 (&p->b, &p->index->pg_root, &p->index->pg_tail);
 
                         // find the largest time index so we can append properly
                         struct adios_index_process_group_struct_v1 * pg;
@@ -414,7 +439,10 @@ START_TIMER (ADIOS_TIMER_POSIX_AD_OPEN);
                                 max_time_index = pg->time_index;
                             pg = pg->next;
                         }
-                        fd->group->time_index = ++max_time_index;
+                        if (fd->mode == adios_mode_append) {
+                            ++max_time_index;
+                        }
+                        fd->group->time_index = max_time_index;
 
                         adios_posix_read_vars_index (&p->b);
                         adios_parse_vars_index_v1 (&p->b, &p->index->vars_root, 
@@ -491,9 +519,10 @@ enum ADIOS_FLAG adios_posix_should_buffer (struct adios_file_struct * fd
         if (s != fd->bytes_written)
         {
             fprintf (stderr, "POSIX method tried to write %llu, "
-                             "only wrote %lld\n"
+                             "only wrote %lld. %s:%d\n"
                     ,fd->bytes_written
                     ,(int64_t)s
+                    ,__func__, __LINE__
                     );
         }
         fd->base_offset += s;
@@ -519,7 +548,7 @@ enum ADIOS_FLAG adios_posix_should_buffer (struct adios_file_struct * fd
 
 void adios_posix_write (struct adios_file_struct * fd
                        ,struct adios_var_struct * v
-                       ,void * data
+                       ,const void * data
                        ,struct adios_method_struct * method
                        )
 {
@@ -534,7 +563,7 @@ void adios_posix_write (struct adios_file_struct * fd
         {
             if (v->free_data == adios_flag_yes)
             {
-                free (v->data);
+                free (v->adata);
                 adios_method_buffer_free (v->data_size);
             }
         }
@@ -555,9 +584,10 @@ void adios_posix_write (struct adios_file_struct * fd
         if (s != fd->bytes_written)
         {
             fprintf (stderr, "POSIX method tried to write %llu, "
-                             "only wrote %lld\n"
+                             "only wrote %lld. %s:%d\n"
                     ,fd->bytes_written
                     ,(int64_t)s
+                    ,__func__, __LINE__
                     );
         }
         fd->base_offset += s;
@@ -608,9 +638,10 @@ void adios_posix_write (struct adios_file_struct * fd
         if (s != var_size)
         {
             fprintf (stderr, "POSIX method tried to write %llu, "
-                             "only wrote %lld\n"
+                             "only wrote %lld. %s:%d\n"
                     ,var_size
                     ,(int64_t)s
+                    ,__func__, __LINE__
                     );
         }
         fd->base_offset += s;
@@ -638,10 +669,10 @@ void adios_posix_get_write_buffer (struct adios_file_struct * fd
         return;
     }
 
-    if (v->data && v->free_data)
+    if (v->adata && v->free_data)
     {
         adios_method_buffer_free (v->data_size);
-        free (v->data);
+        free (v->adata);
     }
 
     mem_allowed = adios_method_buffer_alloc (*size);
@@ -689,7 +720,7 @@ void adios_posix_read (struct adios_file_struct * fd
                       ,struct adios_method_struct * method
                       )
 {
-    v->data = buffer;
+    v->data = v->adata = buffer;
     v->data_size = buffer_size;
 }
 
@@ -765,6 +796,7 @@ static void adios_posix_do_read (struct adios_file_struct * fd
     {
         case 1:
         case 2:
+        case 3:
         {
             struct adios_index_struct_v1 * index = adios_alloc_index_v1(0); // no hashtables
             struct adios_index_process_group_struct_v1 * pg_root = index->pg_root;
@@ -774,7 +806,7 @@ static void adios_posix_do_read (struct adios_file_struct * fd
             adios_parse_index_offsets_v1 (&p->b);
 
             adios_posix_read_process_group_index (&p->b);
-            adios_parse_process_group_index_v1 (&p->b, &pg_root);
+            adios_parse_process_group_index_v1 (&p->b, &pg_root, NULL);
 #if 1
             adios_posix_read_vars_index (&p->b);
             adios_parse_vars_index_v1 (&p->b, &index->vars_root, NULL, NULL);
@@ -837,7 +869,7 @@ static void adios_posix_do_read (struct adios_file_struct * fd
 
                 if (v1)
                 {
-                    var_payload.payload = v1->data;
+                    var_payload.payload = v1->adata;
                     adios_parse_var_data_payload_v1 (&p->b, &var_header
                                                     ,&var_payload
                                                     ,v1->data_size
@@ -910,9 +942,10 @@ void adios_posix_close (struct adios_file_struct * fd
                 if (s != fd->vars_start)
                 {
                     fprintf (stderr, "POSIX method tried to write %llu, "
-                                     "only wrote %lld\n"
+                                     "only wrote %lld. %s:%d\n"
                             ,fd->vars_start
                             ,(int64_t)s
+                            ,__func__, __LINE__
                             );
                 }
                 fd->offset = 0;
@@ -941,9 +974,10 @@ void adios_posix_close (struct adios_file_struct * fd
                         if (s != fd->bytes_written)
                         {
                             fprintf (stderr, "POSIX method tried to write %llu, "
-                                    "only wrote %lld\n"
+                                    "only wrote %lld. %s:%d\n"
                                     ,fd->bytes_written
                                     ,(int64_t)s
+                                    ,__func__, __LINE__
                                     );
                         }
                         fd->base_offset += s;
@@ -968,9 +1002,10 @@ void adios_posix_close (struct adios_file_struct * fd
                 if (s != p->vars_header_size)
                 {
                     fprintf (stderr, "POSIX method tried to write %llu, "
-                                     "only wrote %lld\n"
+                                     "only wrote %lld. %s:%d\n"
                             ,p->vars_header_size
                             ,(int64_t)s
+                            ,__func__, __LINE__
                             );
                 }
                 fd->offset = 0;
@@ -994,7 +1029,7 @@ void adios_posix_close (struct adios_file_struct * fd
             adios_posix_do_write (fd, method, buffer, buffer_offset); // Buffered vars written here
             STOP_TIMER (ADIOS_TIMER_POSIX_IO);
 #ifdef HAVE_MPI
-            if (p->group_comm != MPI_COMM_SELF)
+            if (p->group_comm != MPI_COMM_SELF && p->g_have_mdf)
             {
                 if (p->rank == 0)
                 {
@@ -1035,9 +1070,7 @@ void adios_posix_close (struct adios_file_struct * fd
                         p->b.length = index_sizes [i];
                         p->b.offset = 0;
 
-                        adios_parse_process_group_index_v1 (&p->b
-                                                           ,&new_pg_root
-                                                           );
+                        adios_parse_process_group_index_v1 (&p->b, &new_pg_root, NULL);
                         adios_parse_vars_index_v1 (&p->b, &new_vars_root, NULL, NULL);
                         // do not merge attributes from other processes from 1.4
                         /*
@@ -1047,7 +1080,7 @@ void adios_posix_close (struct adios_file_struct * fd
                         */
 
                         adios_merge_index_v1 (p->index, new_pg_root, 
-                                              new_vars_root, new_attrs_root);
+                                              new_vars_root, new_attrs_root, 0);
                         new_pg_root = 0;
                         new_vars_root = 0;
                         new_attrs_root = 0;
@@ -1084,13 +1117,15 @@ void adios_posix_close (struct adios_file_struct * fd
                     if (s != global_index_buffer_offset)
                     {
                         fprintf (stderr, "POSIX method tried to write %llu, "
-                                         "only wrote %lld\n"
+                                         "only wrote %lld. %s:%d\n"
                                          ,fd->bytes_written
                                          ,(int64_t)s
+                                         ,__func__, __LINE__
                                 );
                     }
 
                     close (p->mf);
+                    free (global_index_buffer);
                 }
                 else
                 {
@@ -1135,9 +1170,10 @@ void adios_posix_close (struct adios_file_struct * fd
                 if (s != fd->vars_start)
                 {
                     fprintf (stderr, "POSIX method tried to write %llu, "
-                                     "only wrote %lld\n"
+                                     "only wrote %lld. %s:%d\n"
                             ,fd->vars_start
                             ,(int64_t)s
+                            ,__func__, __LINE__
                             );
                 }
                 fd->offset = 0;
@@ -1163,9 +1199,10 @@ void adios_posix_close (struct adios_file_struct * fd
                         if (s != fd->bytes_written)
                         {
                             fprintf (stderr, "POSIX method tried to write %llu, "
-                                    "only wrote %lld\n"
+                                    "only wrote %lld. %s:%d\n"
                                     ,fd->bytes_written
                                     ,(int64_t)s
+                                    ,__func__, __LINE__
                                     );
                         }
                         fd->base_offset += s;
@@ -1190,9 +1227,10 @@ void adios_posix_close (struct adios_file_struct * fd
                 if (s != p->vars_header_size)
                 {
                     fprintf (stderr, "POSIX method tried to write %llu, "
-                                     "only wrote %lld\n"
+                                     "only wrote %lld. %s:%d\n"
                             ,p->vars_header_size
                             ,(int64_t)s
+                            ,__func__, __LINE__
                             );
                 }
                 fd->offset = 0;
@@ -1209,7 +1247,7 @@ void adios_posix_close (struct adios_file_struct * fd
             adios_write_index_v1 (&buffer, &buffer_size, &buffer_offset
                                  ,index_start, p->index);
 #ifdef HAVE_MPI
-            if (p->group_comm != MPI_COMM_SELF)
+            if (p->group_comm != MPI_COMM_SELF && p->g_have_mdf)
             {
                 if (p->rank == 0)
                 {
@@ -1251,9 +1289,7 @@ void adios_posix_close (struct adios_file_struct * fd
                         p->b.length = index_sizes [i];
                         p->b.offset = 0;
 
-                        adios_parse_process_group_index_v1 (&p->b
-                                                           ,&new_pg_root
-                                                           );
+                        adios_parse_process_group_index_v1 (&p->b, &new_pg_root, NULL);
                         adios_parse_vars_index_v1 (&p->b, &new_vars_root, NULL, NULL);
                         // do not merge attributes from other processes from 1.4
                         /*
@@ -1262,19 +1298,16 @@ void adios_posix_close (struct adios_file_struct * fd
                                                         );
                          */
 
-                        adios_merge_index_v1 (p->index,new_pg_root, 
-                                              new_vars_root, new_attrs_root);
+                        // global index would become unsorted on main aggregator during merging 
+                        // so sort timesteps in this case (appending)
+                        adios_merge_index_v1 (p->index, new_pg_root, 
+                                              new_vars_root, new_attrs_root, 1);
                     
                         new_pg_root = 0;
                         new_vars_root = 0;
                         new_attrs_root = 0;
                     }
 
-                    adios_sort_index_v1 (&p->index->pg_root
-                                        ,&p->index->vars_root
-                                        ,&p->index->attrs_root
-                                        );
-
                     p->b.buff = buffer_save;
                     p->b.length = buffer_size_save;
                     p->b.offset = offset_save;
@@ -1307,9 +1340,10 @@ void adios_posix_close (struct adios_file_struct * fd
                     if (s != global_index_buffer_offset)
                     {
                         fprintf (stderr, "POSIX method tried to write %llu, "
-                                         "only wrote %lld, Mode: a\n"
+                                         "only wrote %lld, Mode: a. %s:%d\n"
                                          ,global_index_buffer_offset
                                          ,(int64_t)s
+                                         ,__func__, __LINE__
                                 );
                     }
 
@@ -1350,7 +1384,7 @@ void adios_posix_close (struct adios_file_struct * fd
             struct adios_var_struct * v = fd->group->vars;
             while (v)
             {
-                v->data = 0;
+                v->data = v->adata = 0;
                 v = v->next;
             }
 
