@@ -61,6 +61,8 @@ void print_attributes_index
 //const char * value_to_string (enum ADIOS_DATATYPES type, void * data);
 const char * value_to_string_ptr (enum ADIOS_DATATYPES type, void * data, uint64_t element);
 
+int have_subfiles = 0; // global flag indicating if we variables are stored in subfiles (bpdump does not process subfiles)
+
 int main (int argc, char ** argv)
 {
     char * filename;
@@ -112,6 +114,7 @@ int main (int argc, char ** argv)
         dump.dump_var = 0;
     }
 
+    have_subfiles = 0;
     struct adios_bp_buffer_struct_v1 * b = 0;
     uint32_t version = 0;
 
@@ -159,7 +162,7 @@ int main (int argc, char ** argv)
     */
 
     adios_posix_read_process_group_index (b);
-    adios_parse_process_group_index_v1 (b, &pg_root);
+    adios_parse_process_group_index_v1 (b, &pg_root, NULL);
     print_process_group_index (pg_root);
 
     printf (DIVIDER);
@@ -181,10 +184,27 @@ int main (int argc, char ** argv)
     }
 
 
-    uint64_t element_num = 1;
+    uint64_t pg_num = 0;
     pg = pg_root;
     while (pg)
     {
+
+        pg_num++;
+        /* Avoid processing PG's whose offset is beyond the start of index (they are actually in subfiles) */
+        /* Note: only variables have subfile index, PG's don't so we need to be this indirect in the test */
+        if (pg->offset_in_file >= b->pg_index_offset) 
+        {
+            printf ("Process Group %llu offset is beyond the footer.", pg_num);
+            if (have_subfiles) {
+                printf (" It is probably in a subfile but bpdump does not process subfiles.\n");
+            } else {
+                printf (" Since there are no subfiles, this probably means the BP file is corrupt.\n"
+                        "offset=%llu  footer starts at %llu\n", pg->offset_in_file, b->pg_index_offset);
+            }
+            pg = pg->next;
+            continue;
+        }
+
         int var_dims_count = 0;
         struct var_dim * var_dims = 0;
 
@@ -213,7 +233,7 @@ int main (int argc, char ** argv)
 
         adios_posix_read_process_group (b);
         adios_parse_process_group_header_v1 (b, &pg_header);
-        print_process_group_header (element_num++, &pg_header);
+        print_process_group_header (pg_num, &pg_header);
         //printf ("\tSize of group in fil: %llu bytes\n",  b->read_pg_size);
 
         adios_parse_vars_header_v1 (b, &vars_header);
@@ -561,7 +581,7 @@ void print_var_header (struct adios_var_header_struct_v1 * var_header)
             printf ("\t\t\tDim %d l:g:o: ", i++);
             if (d->dimension.var_id == 0)
             {
-                if (d->dimension.time_index == adios_flag_yes)
+                if (d->dimension.is_time_index == adios_flag_yes)
                 {
                     printf ("TIME");
                 }
@@ -581,7 +601,7 @@ void print_var_header (struct adios_var_header_struct_v1 * var_header)
             {
                 if (d->global_dimension.var_id == 0)
                 {
-                    if (d->global_dimension.time_index == adios_flag_yes)
+                    if (d->global_dimension.is_time_index == adios_flag_yes)
                     {
                         printf (":TIME");
                     }
@@ -596,7 +616,7 @@ void print_var_header (struct adios_var_header_struct_v1 * var_header)
                 }
                 if (d->local_offset.var_id == 0)
                 {
-                    if (d->local_offset.time_index == adios_flag_yes)
+                    if (d->local_offset.is_time_index == adios_flag_yes)
                     {
                         printf (":TIME");
                     }
@@ -898,7 +918,7 @@ void print_var_payload (struct adios_var_header_struct_v1 * var_header
                 }
                 else
                 {
-                    if (d->dimension.time_index == adios_flag_yes)
+                    if (d->dimension.is_time_index == adios_flag_yes)
                     {
                         *dims_t = 1;
                     }
@@ -970,6 +990,7 @@ void print_attrs_header (
 
 void print_attribute (struct adios_attribute_struct_v1 * attribute)
 {
+    int i;
     printf ("\t\tAttribute Name (ID): %s (%d)\n"
            ,attribute->name, attribute->id
            );
@@ -981,10 +1002,20 @@ void print_attribute (struct adios_attribute_struct_v1 * attribute)
     else
     {
         printf ("\t\tDatatype: %s\n", adios_type_to_string_int (attribute->type));
-        printf ("\t\tValue: %s\n", bp_value_to_string (attribute->type
-                                                   ,attribute->value
-                                                   )
-               );
+        printf ("\t\t# of elements:   %d\n", attribute->nelems);
+        printf ("\t\tLength in bytes: %d\n", attribute->length);
+        char * p = (char *) attribute->value;
+        int elemsize = (int) adios_get_type_size (attribute->type, attribute->value);
+        if (attribute->nelems == 1) {
+            printf ("\t\tValue: %s\n", bp_value_to_string (attribute->type ,attribute->value));
+        } else {
+            printf ("\t\tValues: %s", bp_value_to_string (attribute->type, p));
+            for (i=1; i<attribute->nelems; i++) {
+                p += elemsize;
+                printf (", %s", bp_value_to_string (attribute->type, p));
+            }
+            printf ("\n");
+        }
     }
 }
 
@@ -1034,6 +1065,10 @@ void print_vars_index (struct adios_index_var_struct_v1 * vars_root)
             printf ("\tFile Index(%d)", vars_root->characteristics [i].file_index);
             printf ("\tTime Index(%d)", vars_root->characteristics [i].time_index);
 
+            if (vars_root->characteristics [i].file_index != (uint32_t)-1) { 
+                have_subfiles = 1;
+            }
+
     		/* NCSU - Print min, max */
 			if (vars_root->type == adios_complex || vars_root->type == adios_double_complex)
 			{
@@ -1233,10 +1268,19 @@ void print_attributes_index
 
             if (attrs_root->characteristics [i].value)
             {
-                printf ("\t\tValue(%s)", bp_value_to_string (attrs_root->type
-                                        ,attrs_root->characteristics [i].value
-                                        )
-                       );
+                if (attrs_root->nelems == 1) {
+                    printf ("\t\tValue(%s)", bp_value_to_string (attrs_root->type, attrs_root->characteristics [i].value));
+                } else {
+                    char * p = (char *) attrs_root->characteristics [i].value;
+                    int elemsize = (int) adios_get_type_size (attrs_root->type, p);
+                    int k;
+                    printf ("\t\tValues(%s", bp_value_to_string (attrs_root->type, p));
+                    for (k=1; k<attrs_root->nelems; k++) {
+                        p += elemsize;
+                        printf (", %s", bp_value_to_string (attrs_root->type, p));
+                    }
+                    printf (")\n");
+                }
             }
             if (attrs_root->characteristics [i].var_id)
             {
