@@ -206,7 +206,7 @@ int main (int argc, char ** argv)
         }
         if (verbose)
             printf ("Process subfiles from %d to %d with thread %d\n", 
-                    targs[tid].startidx, targs[tid].endidx, targs[tid]);
+                    targs[tid].startidx, targs[tid].endidx, targs[tid].tid);
 
         if (tid < nthreads-1) {
             /* Start worker thread. */
@@ -256,7 +256,7 @@ int main (int argc, char ** argv)
         adios_merge_index_v1 (globalindex, 
                               subindex[tid]->pg_root, 
                               subindex[tid]->vars_root, 
-                              subindex[tid]->attrs_root); 
+                              subindex[tid]->attrs_root, 1); 
     }
     write_index (globalindex, filename);
 
@@ -377,7 +377,7 @@ int process_subfiles (int tid, int startidx, int endidx)
          */
 
         adios_posix_read_process_group_index (b[idx]);
-        adios_parse_process_group_index_v1 (b[idx], &new_pg_root);
+        adios_parse_process_group_index_v1 (b[idx], &new_pg_root, NULL);
         print_pg_index (tid, new_pg_root);
 
         adios_posix_read_vars_index (b[idx]);
@@ -388,7 +388,7 @@ int process_subfiles (int tid, int startidx, int endidx)
         adios_parse_attributes_index_v1 (b[idx], &new_attrs_root);
         print_attribute_index (tid, new_attrs_root);
 
-        adios_merge_index_v1 (subindex[tid], new_pg_root, new_vars_root, new_attrs_root); 
+        adios_merge_index_v1 (subindex[tid], new_pg_root, new_vars_root, new_attrs_root, 1); 
 
         adios_posix_close_internal (b[idx]);
         adios_shared_buffer_free (b[idx]);
@@ -546,9 +546,6 @@ void print_variable_index (int tid, struct adios_index_var_struct_v1 * vars_root
                     if (vars_root->type != adios_string)
                         printf ("\tValue(%s)", bp_value_to_string (vars_root->type,
                                     vars_root->characteristics [i].value));
-                    else
-                        printf ("\tValue(\"%s\")", bp_value_to_string (vars_root->type,
-                                    vars_root->characteristics [i].value));
                 }
                 if (vars_root->characteristics [i].dims.count != 0)
                 {
